@@ -4,6 +4,8 @@ import enum
 
 INDEX_PRICE_EXTRA_TRADE_DAYS = 15
 FUND_NAV_EXTRA_TRADE_DAYS = 5
+QUARTER_UPDATE_DATE_LIST = ('0331', '0630', '0930', '1231')
+SEMI_UPDATE_DATE_LIST = ('0630', '1231')
 
 
 class ReadableEnum(object):
@@ -96,6 +98,21 @@ class CodeFeeMode(enum.IntEnum):
     b_type = 2  # 后端收费
 
 
+# 股票因子类型
+class StockFactorType(enum.IntEnum):
+    BASIC = 1  # 基础
+    VALUE = 2  # 价值
+    GROWTH = 3  # 成长
+    FIN_QUALITY = 4  # 财务质量
+    LEVERAGE = 5  # 杠杆
+    SCALE = 6  # 规模
+    MOMENTUM = 7  # 动量
+    VOLATILITY = 8  # 波动率
+    LIQUIDITY = 9  # 流动性
+    ANALYST = 10  # 分析师
+    TECH = 11  # 技术
+
+
 class ExchangeID(ReadableEnum):
     NOT_AVAILABLE = 0
     SSE = 1
