@@ -450,6 +450,21 @@ class BasicDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {Fund_size_and_hold_rate.__tablename__}')
                 return pd.DataFrame([])
 
+    def delete_fund_size_hold_rate(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    Fund_size_and_hold_rate
+                ).filter(
+                    Fund_size_and_hold_rate.fund_id.in_(fund_list),
+                    Fund_size_and_hold_rate.datetime == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {Fund_size_and_hold_rate.__tablename__}')
+                return False
+
     @staticmethod
     def get_fund_size_range(start_date='', end_date='', fund_list: Tuple[str] = ()):
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -531,6 +546,21 @@ class BasicDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {FundHoldAsset.__tablename__}')
                 return pd.DataFrame([])
 
+    def delete_fund_hold_asset(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    FundHoldAsset
+                ).filter(
+                    FundHoldAsset.fund_id.in_(fund_list),
+                    FundHoldAsset.datetime == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {FundHoldAsset.__tablename__}')
+                return False
+
     @staticmethod
     def get_fund_hold_industry_by_id(fund_id):
         with BasicDatabaseConnector().managed_session() as db_session:
@@ -655,6 +685,21 @@ class BasicDataApi(metaclass=Singleton):
                 print(f'Failed to get time range data <err_msg> {e} from {FundHoldIndustry.__tablename__}')
                 return
 
+    def delete_fund_hold_industry(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    FundHoldIndustry
+                ).filter(
+                    FundHoldIndustry.fund_id.in_(fund_list),
+                    FundHoldIndustry.datetime == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {FundHoldIndustry.__tablename__}')
+                return False
+
     def get_fund_hold_stock(self, dt=None) -> pd.DataFrame:
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
@@ -690,6 +735,21 @@ class BasicDataApi(metaclass=Singleton):
                 print(f'Failed to get time range data <err_msg> {e} from {FundHoldBond.__tablename__}')
                 return
 
+    def delete_fund_hold_bond(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with BasicDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    FundHoldBond
+                ).filter(
+                    FundHoldBond.fund_id.in_(fund_list),
+                    FundHoldBond.datetime == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {FundHoldBond.__tablename__}')
+                return False
+
     def delete_fund_rate(self, date_to_delete: datetime.date, fund_list: List[str]):
         with BasicDatabaseConnector().managed_session() as db_session:
             try:
