@@ -106,18 +106,6 @@ class DerivedDataApi(metaclass=Singleton):
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {FundIndicatorGroup.__tablename__}')
 
-    def get_fund_score(self):
-        with DerivedDatabaseConnector().managed_session() as quant_session:
-            try:
-                query = quant_session.query(
-                        FundScore,
-                    )
-                tag_df = pd.read_sql(query.statement, query.session.bind)
-                return tag_df
-
-            except Exception as e:
-                print(f'Failed to get data <err_msg> {e} from {FundScore.__tablename__}')
-
     def get_fund_score_extended_by_id(self, fund_id, data_cycle: str = '1Y'):
         with DerivedDatabaseConnector().managed_session() as quant_session:
             try:
@@ -492,3 +480,29 @@ class DerivedDataApi(metaclass=Singleton):
                 print(f'Failed to get_market_portfolio_position {e} from {ThirdPartyPortfolioPositionLatest.__tablename__}')
                 return pd.DataFrame([])
 
+    def get_market_portfolio_position_by_src(self, po_srcs: Tuple[int] = ()):
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                query = session.query(
+                    ThirdPartyPortfolioPositionLatest
+                )
+                if po_srcs:
+                    query = query.filter(
+                        ThirdPartyPortfolioPositionLatest.po_src.in_(po_srcs),
+                    )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get_market_portfolio_position_by_src {e} from {ThirdPartyPortfolioPositionLatest.__tablename__}')
+                return
+
+    def get_stock_factor_info(self):
+        with DerivedDatabaseConnector().managed_session() as session:
+            try:
+                query = session.query(
+                    StockFactorInfo
+                )
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get_stock_factor_info {e} from {ThirdPartyPortfolioPositionLatest.__tablename__}')
+                return
+
