@@ -1,3 +1,5 @@
+
+import datetime
 from typing import List, Tuple, Union
 import pandas as pd
 from sqlalchemy.sql import func
@@ -451,7 +453,7 @@ class RawDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {EmFundScale.__tablename__}')
                 return None
 
-    def get_em_stock_price(self, start_date: str, end_date: str, stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
+    def get_em_stock_price(self, start_date: str = '', end_date: str = '', stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 if columns:
@@ -463,20 +465,24 @@ class RawDataApi(metaclass=Singleton):
                     query = db_session.query(
                         EmStockPrice
                     )
+                if start_date:
+                    query = query.filter(
+                        EmStockPrice.DATES >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EmStockPrice.DATES <= end_date,
+                    )
                 if stock_list:
                     query = query.filter(
                         EmStockPrice.CODES.in_(stock_list)
                     )
-                query = query.filter(
-                    EmStockPrice.DATES >= start_date,
-                    EmStockPrice.DATES <= end_date,
-                )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {EmStockPrice.__tablename__}')
                 return None
 
-    def get_em_stock_post_price(self, start_date: str, end_date: str, stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
+    def get_em_stock_post_price(self, start_date: str = '', end_date: str = '', stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
                 if columns:
@@ -492,10 +498,14 @@ class RawDataApi(metaclass=Singleton):
                     query = query.filter(
                         EmStockPostPrice.CODES.in_(stock_list)
                     )
-                query = query.filter(
-                    EmStockPostPrice.DATES >= start_date,
-                    EmStockPostPrice.DATES <= end_date,
-                )
+                if start_date:
+                    query = query.filter(
+                        EmStockPostPrice.DATES >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EmStockPostPrice.DATES <= end_date,
+                    )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {EmStockPostPrice.__tablename__}')
@@ -516,34 +526,47 @@ class RawDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {EmStockInfo.__tablename__}')
                 return None
 
-    def get_em_daily_info(self, start_date: str, end_date: str, stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
+    def get_em_daily_info(self, start_date: str = '', end_date: str = '', stock_list: Tuple[str] = (), columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
-                query = db_session.query(
-                    EmStockDailyInfo.CODES,
-                    EmStockDailyInfo.DATES,
-                )
                 if columns:
-                    query = query.add_columns(*columns)
+                    query = db_session.query(
+                        EmStockDailyInfo.CODES,
+                        EmStockDailyInfo.DATES,
+                    ).add_columns(*columns)
+                else:
+                    query = db_session.query(
+                        EmStockDailyInfo
+                    )
                 if stock_list:
                     query = query.filter(
                         EmStockDailyInfo.CODES.in_(stock_list)
                     )
-                query = query.filter(
-                    EmStockDailyInfo.DATES >= start_date,
-                    EmStockDailyInfo.DATES <= end_date,
-                )
+                if start_date:
+                    query = query.filter(
+                        EmStockDailyInfo.DATES >= start_date,
+                    )
+                if end_date:
+                    query = query.filter(
+                        EmStockDailyInfo.DATES <= end_date,
+                    )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print(f'Failed to get data <err_msg> {e} from {EmStockDailyInfo.__tablename__}')
                 return None
 
-    def get_em_stock_fin_fac(self, *, stock_list: Tuple[str] = (), date_list: Tuple[str] = (), start_date: str = '', end_date: str = ''):
+    def get_em_stock_fin_fac(self, *, stock_list: Tuple[str] = (), date_list: Tuple[str] = (), start_date: str = '', end_date: str = '', columns: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
-                query = db_session.query(
-                    EmStockFinFac
-                )
+                if not columns:
+                    query = db_session.query(
+                        EmStockFinFac
+                    )
+                else:
+                    query = db_session.query(
+                        EmStockFinFac.CODES,
+                        EmStockFinFac.DATES,
+                    ).add_columns(*columns)
                 if stock_list:
                     query = query.filter(
                         EmStockFinFac.CODES.in_(stock_list)
@@ -660,6 +683,21 @@ class RawDataApi(metaclass=Singleton):
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
+    def delete_em_fund_hold_rate(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    EmFundHoldingRate
+                ).filter(
+                    EmFundHoldingRate.CODES.in_(fund_list),
+                    EmFundHoldingRate.DATES == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {EmFundHoldingRate.__tablename__}')
+                return False
+
     def get_em_fund_list(self, date: str, limit = -1):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -871,7 +909,22 @@ class RawDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {EMFundHoldAsset.__tablename__}')
                 return None
 
-    def get_em_fund_hold_industry(self, end_date: str = '', fund_ids: Tuple[str] = ()):
+    def delete_em_fund_hold_asset(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    EMFundHoldAsset
+                ).filter(
+                    EMFundHoldAsset.CODES.in_(fund_list),
+                    EMFundHoldAsset.DATES == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {EMFundHoldAsset.__tablename__}')
+                return False
+
+    def get_em_fund_hold_industry(self, start_date: str = '', end_date: str = '', fund_ids: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as raw_session:
             try:
                 query = raw_session.query(EMFundHoldIndustry)
@@ -879,6 +932,10 @@ class RawDataApi(metaclass=Singleton):
                     query = query.filter(
                         EMFundHoldIndustry.CODES.in_(fund_ids),
                     )
+                if start_date:
+                    query = query.filter(
+                        EMFundHoldIndustry.DATES >= start_date,
+                    )
                 if end_date:
                     query = query.filter(
                         EMFundHoldIndustry.DATES <= end_date,
@@ -888,6 +945,21 @@ class RawDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {EMFundHoldIndustry.__tablename__}')
                 return None
 
+    def delete_em_fund_hold_industry(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    EMFundHoldIndustry
+                ).filter(
+                    EMFundHoldIndustry.CODES.in_(fund_list),
+                    EMFundHoldIndustry.DATES == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {EMFundHoldIndustry.__tablename__}')
+                return False
+
     def get_em_fund_hold_stock(self, end_date: str = '', fund_ids: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as raw_session:
             try:
@@ -905,7 +977,7 @@ class RawDataApi(metaclass=Singleton):
                 print(f'Failed to get data <err_msg> {e} from {EMFundHoldStock.__tablename__}')
                 return None
 
-    def get_em_fund_hold_bond(self, end_date: str = '', fund_ids: Tuple[str] = ()):
+    def get_em_fund_hold_bond(self, start_date: str = '', end_date: str = '', fund_ids: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as raw_session:
             try:
                 query = raw_session.query(EMFundHoldBond)
@@ -913,15 +985,34 @@ class RawDataApi(metaclass=Singleton):
                     query = query.filter(
                         EMFundHoldBond.CODES.in_(fund_ids),
                     )
+                if start_date:
+                    query = query.filter(
+                        EMFundHoldBond.DATES >= start_date,
+                    )
                 if end_date:
                     query = query.filter(
-                        EMFundHoldBond.DATES <= end_date
+                        EMFundHoldBond.DATES <= end_date,
                     )
                 return pd.read_sql(query.statement, query.session.bind)
             except Exception as e:
                 print('Failed to get data <err_msg> {}'.format(e))
                 return None
 
+    def delete_em_fund_hold_bond(self, date_to_delete: datetime.date, fund_list: List[str]):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                db_session.query(
+                    EMFundHoldBond
+                ).filter(
+                    EMFundHoldBond.CODES.in_(fund_list),
+                    EMFundHoldBond.DATES == date_to_delete,
+                ).delete(synchronize_session=False)
+                db_session.commit()
+                return True
+            except Exception as e:
+                print(f'Failed to delete data <err_msg> {e} from {EMFundHoldBond.__tablename__}')
+                return False
+
     def get_em_mng_info(self, fund_ids: Tuple[str] = ()):
         with RawDatabaseConnector().managed_session() as db_session:
             try:
@@ -1085,3 +1176,43 @@ class RawDataApi(metaclass=Singleton):
                 return True
             except Exception as e:
                 print(f'Failed to delete data <err_msg> {e} from {EmStockIndustrialCapital.__tablename__}')
+
+    def get_em_stock_industrial_capital_trade_detail(self):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EmStockIndustrialCapitalTradeDetail)
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockIndustrialCapitalTradeDetail.__tablename__}')
+
+    def get_em_stock_shsz_to_hk_connect(self):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EmSHSZToHKStockConnect)
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmSHSZToHKStockConnect.__tablename__}')
+
+    def get_em_stock_research_info(self):
+        with RawDatabaseConnector().managed_session() as raw_session:
+            try:
+                query = raw_session.query(EmStockResearchInfo)
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockResearchInfo.__tablename__}')
+
+    def get_em_stock_yearly(self, columns: Tuple[str] = ()):
+        with RawDatabaseConnector().managed_session() as db_session:
+            try:
+                if not columns:
+                    query = db_session.query(
+                        EmStockYearly
+                    )
+                else:
+                    query = db_session.query(
+                        EmStockYearly.CODES,
+                        EmStockYearly.year,
+                    ).add_columns(*columns)
+                return pd.read_sql(query.statement, query.session.bind)
+            except Exception as e:
+                print(f'Failed to get data <err_msg> {e} from {EmStockYearly.__tablename__}')
