@@ -45,7 +45,7 @@ def add_column_to_style_analysis_factor(df: pd.DataFrame, new_column_name: str):
     db_session.close()
 
 
-class BasicDataHelper(object):
+class BasicDataHelper:
     def __init__(self):
         self._updated_count = defaultdict(int)
 
