@@ -6,19 +6,18 @@ import numpy as np
 import pandas as pd
 from typing import List, Dict, Optional, Tuple, Union, Set
 
-from ....constant import SectorType, IndexPriceSource, INDEX_PRICE_EXTRA_TRADE_DAYS, FUND_NAV_EXTRA_TRADE_DAYS
+from ....constant import SectorType, IndexPriceSource
+from ....constant import INDEX_PRICE_EXTRA_TRADE_DAYS, FUND_NAV_EXTRA_TRADE_DAYS, QUARTER_UPDATE_DATE_LIST, SEMI_UPDATE_DATE_LIST
 from ...api.raw import RawDataApi
 from ...api.basic import BasicDataApi
 from ...view.basic_models import *
 from ...manager.fund_tag_update import fund_tag
-# from ..raw.raw_data_helper import RawDataHelper
+from ..raw.raw_data_helper import RawDataHelper
 from .basic_data_helper import BasicDataHelper
 
 
 class BasicDataPart1:
 
-    _REPORT_DATE_LIST = ['0331', '0630', '0930', '1231']
-
     def __init__(self, data_helper: BasicDataHelper):
         self._data_helper = data_helper
         self._raw_data_api = RawDataApi()
@@ -102,24 +101,24 @@ class BasicDataPart1:
                     return '国际(QDII)基金'
             return x.invest_type_class
 
-        def _invest_type_class_to_wind_class_2(invest_type_class: str) -> str:
+        def _invest_type_class_to_wind_class_2(x: pd.Series) -> str:
             '''Choice的基金二级类型转换为wind的'''
 
-            if invest_type_class == '混合债券型基金（一级）':
+            if x.invest_type_class_2 == '混合债券型基金（一级）':
                 return '混合债券型一级基金'
-            elif invest_type_class == '长期纯债型基金':
+            elif x.invest_type_class_2 == '长期纯债型基金':
                 return '中长期纯债型基金'
-            elif invest_type_class == '中短期纯债型基金':
+            elif x.invest_type_class_2 == '中短期纯债型基金':
                 return '短期纯债型基金'
-            elif invest_type_class == '混合债券型基金（二级）':
+            elif x.invest_type_class_2 == '混合债券型基金（二级）':
                 return '混合债券型二级基金'
-            elif invest_type_class == '被动指数型基金':
+            elif x.invest_type_class_1 == '债券型基金' and x.invest_type_class_2 == '被动指数型基金':
                 return '被动指数型债券基金'
-            elif invest_type_class in ('QDII基金', '封闭式基金', '保本型基金'):
+            elif x.invest_type_class_2 in ('QDII基金', '封闭式基金', '保本型基金'):
                 # error case
                 return
             else:
-                return invest_type_class
+                return x.invest_type_class_2
 
         def _lambda_structure_type(x: pd.Series) -> int:
             if x.base_fund_id is None:
@@ -217,7 +216,7 @@ class BasicDataPart1:
         df['fund_id'] = df.order_book_id.transform(lambda x: x + '!0')
         # Choice的基金一二级分类转换为wind的
         df['wind_class_1'] = df[['desc_name', 'invest_type_class_2']].transform(_invest_type_class_to_wind_class_1, axis=1)
-        df['wind_class_2'] = df.invest_type_class_2.transform(_invest_type_class_to_wind_class_2)
+        df['wind_class_2'] = df[['invest_type_class_1', 'invest_type_class_2']].transform(_invest_type_class_to_wind_class_2, axis=1)
         df['asset_type'] = df.apply(lambda x: self._find_tag(x.desc_name, x.wind_class_2), axis=1)
         df['structure_type'] = df.apply(_lambda_structure_type, axis=1)
         df['is_etf'] = df.desc_name.transform(lambda x: 1 if ('ETF' in x and '联接' not in x) else 0)
@@ -581,13 +580,14 @@ class BasicDataPart1:
 
     def fund_status_latest(self, end_date: str):
         try:
-            df: Optional[pd.DataFrame] = self._raw_data_api.get_em_fund_status(end_date=end_date)
+            df: Optional[pd.DataFrame] = self._raw_data_api.get_em_fund_status(end_date=end_date).drop(columns='_update_time')
             if df is None:
                 print('failed to get em fund scale for fund status latest')
                 return False
             # 将场内变为场外的
             df['CODES'] = df.CODES.transform(lambda x: x.split('.')[0] + '.OF' if '.OF' not in x else x)
-            df = df.pivot(index='DATES', columns='CODES', values=['PURCHSTATUS', 'REDEMSTATUS', 'LPLIMIT', 'OTCLPLIMITJG', 'OTCLPLIMITGR', 'TRADESTATUS']).fillna(method='ffill')
+            # pivot on all remaining columns
+            df = df.drop(columns='FUNDSCALE').pivot(index='DATES', columns='CODES').fillna(method='ffill')
             df = df.iloc[[-1], :].stack().reset_index()
             df['fund_id'] = df[['DATES', 'CODES']].apply(lambda x: self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
             df = df[df.fund_id.notna()]
@@ -666,16 +666,7 @@ class BasicDataPart1:
             }
             df = df.rename(columns = name_dic)
             df = df.dropna(subset=['fund_id','datetime'], axis=0)
-            # 寻找离目前最近且没超过当前时间的财报日期，将数据写在这个日期上（先删掉这个日期上的旧数据）
-            end_date_dt = pd.to_datetime(end_date, infer_datetime_format=True)
-            md = end_date_dt.strftime('%m%d')
-            sentry_date = pd.Series(self._REPORT_DATE_LIST)
-            sentry = sentry_date[sentry_date <= md]
-            if sentry.empty:
-                # 这里取前一年的12月31
-                real_date = datetime.date(end_date_dt.year - 1, 12, 31)
-            else:
-                real_date = pd.to_datetime(str(end_date_dt.year)+sentry.array[-1], infer_datetime_format=True).date()
+            real_date = RawDataHelper.get_prev_target_date(end_date, QUARTER_UPDATE_DATE_LIST)
             df['datetime'] = real_date
             # TODO: 最好原子提交下边两步
             BasicDataApi().delete_fund_rate(real_date, df.fund_id.to_list())
@@ -703,65 +694,132 @@ class BasicDataPart1:
             traceback.print_exc()
             return False
 
-    def fund_size_hold_rate_history(self):
-        start_date = '20200630'
-        end_date = '20200630'
+    def fund_size_hold_rate(self, start_date: str, end_date: str, is_history=True):
+        try:
+            if is_history:
+                fund_holding_rate = self._raw_data_api.get_em_fund_holding_rate(start_date=start_date, end_date=end_date)
+            else:
+                real_date = RawDataHelper.get_prev_target_date(end_date, SEMI_UPDATE_DATE_LIST)
+                fund_holding_rate = self._raw_data_api.get_em_fund_holding_rate(start_date=real_date, end_date=real_date)
+            if fund_holding_rate is not None and not fund_holding_rate.empty:
+                fund_holding_rate['fund_id'] = fund_holding_rate.apply(
+                    lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
+                fund_holding_rate = fund_holding_rate[fund_holding_rate.fund_id.notna()]
+                fund_holding_rate = fund_holding_rate.drop(columns=['CODES', '_update_time']).rename(columns={
+                    'DATES': 'datetime',
+                    'HOLDPERSONALHOLDINGPCT': 'personal_holds',
+                    'HOLDINSTIHOLDINGPCT': 'institution_holds',
+                    'HOLDNUM': 'hold_num',
+                })
+                fund_holding_rate = fund_holding_rate[fund_holding_rate.drop(columns='fund_id').notna().any(axis=1)]
 
-        fund_holding_rate = self._raw_data_api.get_em_fund_holding_rate(start_date=start_date, end_date=end_date)
-        fund_holding_rate['fund_id'] = fund_holding_rate.apply(
-            lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
-        fund_holding_rate = fund_holding_rate[fund_holding_rate.fund_id.notna()]
-        fund_holding_rate = fund_holding_rate.drop(['CODES', '_update_time'], axis=1).rename(columns={
-            'DATES': 'datetime',
-            'HOLDPERSONALHOLDINGPCT': 'personal_holds',
-            'HOLDINSTIHOLDINGPCT': 'institution_holds',
-            'HOLDNUM': 'hold_num',
-        })
-        fund_holding_rate = fund_holding_rate[fund_holding_rate.drop(columns='fund_id').notna().any(axis=1)]
+            if is_history:
+                fund_scale = self._raw_data_api.get_em_fund_scale(start_date=start_date, end_date=end_date)
+            else:
+                real_date = RawDataHelper.get_prev_target_date(end_date, QUARTER_UPDATE_DATE_LIST)
+                fund_scale = self._raw_data_api.get_em_fund_scale(start_date=real_date, end_date=real_date)
+            fund_scale = fund_scale.pivot(index='DATES', columns='CODES', values='FUNDSCALE').fillna(method='ffill').stack()
+            fund_scale = pd.DataFrame(fund_scale, columns=['FUNDSCALE']).reset_index()
+            fund_scale['fund_id'] = fund_scale.apply(
+                lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
+            fund_scale = fund_scale[fund_scale.fund_id.notna() & fund_scale.FUNDSCALE.notna()]
+            fund_scale = fund_scale.drop(columns=['CODES']).rename(columns={'DATES': 'datetime', 'FUNDSCALE': 'size'})
 
-        fund_scale = self._raw_data_api.get_em_fund_scale(start_date=start_date, end_date=end_date)
-        fund_scale = fund_scale.pivot(index='DATES', columns='CODES', values='FUNDSCALE').fillna(method='ffill').stack()
-        fund_scale = pd.DataFrame(fund_scale, columns=['FUNDSCALE']).reset_index()
-        fund_scale['fund_id'] = fund_scale.apply(
-            lambda x: self._data_helper._get_fund_id_from_order_book_id(x['CODES'].split('.')[0], x['DATES']), axis=1)
-        fund_scale = fund_scale[fund_scale.fund_id.notna()]
-        fund_scale = fund_scale.drop(['CODES'], axis=1).rename(columns={'DATES': 'datetime', 'FUNDSCALE': 'size'})
-        fund_scale = fund_scale.dropna().drop_duplicates(subset=['size', 'fund_id'], keep='first')
+            if fund_holding_rate is not None and not fund_holding_rate.empty:
+                df = fund_scale.set_index(['fund_id', 'datetime']).join(fund_holding_rate.set_index(['fund_id', 'datetime']), how='left').reset_index()
+                df['datetime'] = df.datetime.dt.date
+            else:
+                df = fund_scale
+            df = df.drop_duplicates(subset='fund_id')
+            if not is_history:
+                now_df = BasicDataApi().get_history_fund_size(start_date=real_date, end_date=real_date)
+                if now_df is not None:
+                    now_df = now_df.astype({'size': 'float64', 'personal_holds': 'float64', 'institution_holds': 'float64', 'hold_num': 'float64'})
+                    # 过滤出来新增基金benchmark以及旧基金的benchmark发生变化的情况
+                    now_df = now_df.drop(columns=['_update_time']).sort_values(by=['fund_id', 'datetime']).drop_duplicates(subset='fund_id', keep='last')
+                    # merge on all columns
+                    df = df.merge(now_df, how='left', indicator=True, validate='one_to_one')
+                    df = df[df._merge == 'left_only'].drop(columns=['_merge'])
+                # TODO: 最好原子提交下边两步
+                BasicDataApi().delete_fund_size_hold_rate(real_date, df.fund_id.to_list())
+            self._data_helper._upload_basic(df, Fund_size_and_hold_rate.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
+            return False
 
-        df = fund_scale.set_index(['fund_id', 'datetime']).join(fund_holding_rate.set_index(['fund_id', 'datetime']), how='outer').reset_index()
-        self._data_helper._upload_basic(df, Fund_size_and_hold_rate.__table__.name)
+    def fund_hold_asset(self, start_date: str, end_date: str, is_history=True):
+        try:
+            if is_history:
+                # history, update only on report date
+                df = self._raw_data_api.get_em_fund_hold_asset(start_date, end_date)
+            else:
+                real_date = RawDataHelper.get_prev_target_date(end_date, QUARTER_UPDATE_DATE_LIST)
+                df = self._raw_data_api.get_em_fund_hold_asset(real_date, real_date)
+            df['CODES'] = df.apply(lambda x: self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
+            df = df.drop(columns='_update_time').dropna(subset=['CODES']).drop_duplicates(subset='CODES')
+            name_dict = {
+                'CODES': 'fund_id',
+                'DATES': 'datetime',
+                'PRTSTOCKTONAV': 'stock_nav_ratio',
+                'PRTBONDTONAV': 'bond_nav_ratio',
+                'PRTCASHTONAV': 'cash_nav_ratio',
+                'PRTOTHERTONAV': 'other_nav_ratio',
+                'MMFFIRSTREPOTONAV': 'first_repo_to_nav',
+                'MMFAVGPTM': 'avg_ptm',
+            }
+            df = df.rename(columns=name_dict)
+            if not is_history:
+                now_df = BasicDataApi().get_fund_hold_asset(dt=real_date)
+                if now_df is not None:
+                    df = df.replace({None: np.nan}).infer_objects()
+                    now_df = now_df.replace({None: np.nan}).infer_objects()
+                    # 过滤出来新增基金benchmark以及旧基金的benchmark发生变化的情况
+                    now_df = now_df.drop(columns=['_update_time']).sort_values(by=['fund_id', 'datetime']).drop_duplicates(subset='fund_id', keep='last')
+                    # merge on all columns
+                    df = df.merge(now_df, how='left', indicator=True, validate='one_to_one')
+                    df = df[df._merge == 'left_only'].drop(columns=['_merge'])
+                # TODO: 最好原子提交下边两步
+                BasicDataApi().delete_fund_hold_asset(real_date, df.fund_id.to_list())
+            self._data_helper._upload_basic(df, FundHoldAsset.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
+            return False
 
-    def fund_hold_asset(self, start_date: str, end_date: str):
-        # history, update only on report date
-        em_hold_asset = self._raw_data_api.get_em_fund_hold_asset(start_date, end_date)
-        em_hold_asset['CODES'] = em_hold_asset.apply(lambda x:
-                            self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
-        em_hold_asset = em_hold_asset.dropna(subset=['CODES']).drop(columns='_update_time')
-        name_dict = {
-            'CODES': 'fund_id',
-            'DATES': 'datetime',
-            'PRTSTOCKTONAV': 'stock_nav_ratio',
-            'PRTBONDTONAV': 'bond_nav_ratio',
-            'PRTCASHTONAV': 'cash_nav_ratio',
-            'PRTOTHERTONAV': 'other_nav_ratio',
-            'MMFFIRSTREPOTONAV': 'first_repo_to_nav',
-            'MMFAVGPTM': 'avg_ptm',
-        }
-        em_hold_asset = em_hold_asset.rename(columns=name_dict)
-        self._data_helper._upload_basic(em_hold_asset, FundHoldAsset.__table__.name)
-
-    def fund_hold_industry(self, end_date: str):
-        # history, update only on report date
-        em_hold_industry = self._raw_data_api.get_em_fund_hold_industry(end_date)
-        em_hold_industry['CODES'] = em_hold_industry.apply(lambda x:
-                            self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
-        em_hold_industry = em_hold_industry.dropna(subset=['CODES']).drop(columns='_update_time')
-        name_dict = {
-            'CODES': 'fund_id',
-            'DATES': 'datetime',
-        }
-        em_hold_industry = em_hold_industry.rename(columns=name_dict)
-        self._data_helper._upload_basic(em_hold_industry, FundHoldIndustry.__table__.name)
+    def fund_hold_industry(self, start_date: str, end_date: str, is_history=False):
+        try:
+            if is_history:
+                # history, update only on report date
+                df = self._raw_data_api.get_em_fund_hold_industry(start_date, end_date)
+            else:
+                real_date = RawDataHelper.get_prev_target_date(end_date, QUARTER_UPDATE_DATE_LIST)
+                df = self._raw_data_api.get_em_fund_hold_industry(real_date, real_date)
+            df['CODES'] = df.apply(lambda x: self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
+            df = df.dropna(subset=['CODES']).drop(columns='_update_time')
+            name_dict = {
+                'CODES': 'fund_id',
+                'DATES': 'datetime',
+            }
+            df = df.rename(columns=name_dict)
+            if not is_history:
+                now_df = BasicDataApi().get_fund_hold_industry(dt=real_date)
+                if now_df is not None:
+                    # 过滤出来新增基金benchmark以及旧基金的benchmark发生变化的情况
+                    now_df = now_df.drop(columns=['_update_time']).sort_values(by=['fund_id', 'datetime']).drop_duplicates(subset='fund_id', keep='last')
+                    # merge on all columns
+                    df = df.merge(now_df, how='left', indicator=True, validate='one_to_one')
+                    df = df[df._merge == 'left_only'].drop(columns=['_merge'])
+                # TODO: 最好原子提交下边两步
+                BasicDataApi().delete_fund_hold_industry(real_date, df.fund_id.to_list())
+            self._data_helper._upload_basic(df, FundHoldIndustry.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
+            return False
 
     def fund_hold_stock(self, end_date: str):
         #history, update only on report date
@@ -776,18 +834,37 @@ class BasicDataPart1:
         em_hold_stock = em_hold_stock.rename(columns=name_dict)
         self._data_helper._upload_basic(em_hold_stock, FundHoldStock.__table__.name)
 
-    def fund_hold_bond(self, end_date: str):
-        #history, update only on report date
-        em_hold_bond = self._raw_data_api.get_em_fund_hold_bond(end_date)
-        em_hold_bond['CODES'] = em_hold_bond.apply(lambda x :
-                            self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
-        em_hold_bond = em_hold_bond.dropna(subset=['CODES']).drop('_update_time', axis=1)
-        name_dict = {
-            'CODES':'fund_id',
-            'DATES':'datetime',
-        }
-        em_hold_bond = em_hold_bond.rename(columns=name_dict)
-        self._data_helper._upload_basic(em_hold_bond, FundHoldBond.__table__.name)
+    def fund_hold_bond(self, start_date: str, end_date: str, is_history=True):
+        try:
+            if is_history:
+                # history, update only on report date
+                df = self._raw_data_api.get_em_fund_hold_bond(start_date, end_date)
+            else:
+                real_date = RawDataHelper.get_prev_target_date(end_date, QUARTER_UPDATE_DATE_LIST)
+                df = self._raw_data_api.get_em_fund_hold_bond(real_date, real_date)
+            df['CODES'] = df.apply(lambda x: self._data_helper._get_fund_id_from_order_book_id(x.CODES.split('.')[0], x.DATES), axis=1)
+            df = df.dropna(subset=['CODES']).drop(columns='_update_time')
+            name_dict = {
+                'CODES': 'fund_id',
+                'DATES': 'datetime',
+            }
+            df = df.rename(columns=name_dict)
+            if not is_history:
+                now_df = BasicDataApi().get_fund_hold_bond(dt=real_date)
+                if now_df is not None:
+                    # 过滤出来新增基金benchmark以及旧基金的benchmark发生变化的情况
+                    now_df = now_df.drop(columns=['_update_time']).sort_values(by=['fund_id', 'datetime']).drop_duplicates(subset='fund_id', keep='last')
+                    # merge on all columns
+                    df = df.merge(now_df, how='left', indicator=True, validate='one_to_one')
+                    df = df[df._merge == 'left_only'].drop(columns=['_merge'])
+                # TODO: 最好原子提交下边两步
+                BasicDataApi().delete_fund_hold_bond(real_date, df.fund_id.to_list())
+            self._data_helper._upload_basic(df, FundHoldBond.__table__.name)
+            return True
+        except Exception as e:
+            print(e)
+            traceback.print_exc()
+            return False
 
     def mng_info(self):
         df = self._raw_data_api.get_em_mng_info()
@@ -846,9 +923,9 @@ class BasicDataPart1:
         #     if not self.fund_info_from_raw(new_fund_list):
         #         failed_tasks.append('fund_info')
 
-        # TODO: Depend on RQ, change to EM
-        if not self.fund_rating_latest():
-            failed_tasks.append('fund_rating_latest')
+        # Depend on RQ
+        # if not self.fund_rating_latest():
+        #     failed_tasks.append('fund_rating_latest')
 
         if not self.fund_size(start_date, end_date):
             failed_tasks.append('fund_size')
@@ -865,6 +942,32 @@ class BasicDataPart1:
         if not self.fund_status_latest(end_date):
             failed_tasks.append('fund_status_latest')
 
+        # 获取下一个交易日
+        trading_day_df = BasicDataApi().get_trading_day_list(start_date=end_date)
+        if trading_day_df.shape[0] <= 1:
+            print(f'get trading days start with {end_date} failed')
+            failed_tasks.append('get_trading_day_list for weekly update in basic')
+        else:
+            next_trading_day = trading_day_df.iloc[1, :].datetime
+            print(f'got next trading day {next_trading_day}')
+            end_date_dt = pd.to_datetime(end_date, infer_datetime_format=True).date()
+            next_trading_dt = pd.to_datetime(next_trading_day, infer_datetime_format=True).date()
+            if end_date_dt.weekday() < next_trading_dt.weekday() and next_trading_dt < end_date_dt + datetime.timedelta(weeks=1):
+                # 表明本周后边还有交易日，今天不需要更新
+                print(f'weekly data only update on the last day of week, not today {end_date_dt}')
+            else:
+                if not self.fund_hold_asset(end_date, end_date, is_history=False):
+                    failed_tasks.append('fund_hold_asset')
+
+                if not self.fund_size_hold_rate(end_date, end_date, is_history=False):
+                    failed_tasks.append('fund_size_hold_rate')
+
+                if not self.fund_hold_bond(end_date, end_date, is_history=False):
+                    failed_tasks.append('fund_bond')
+
+                if not self.fund_hold_industry(end_date, end_date, is_history=False):
+                    failed_tasks.append('fund_industry')
+
         return failed_tasks
 
 if __name__ == '__main__':
