@@ -1,5 +1,7 @@
 
 import time
+import json
+import datetime
 from typing import Union, Tuple
 
 import numpy as np
@@ -10,28 +12,33 @@ from sklearn.preprocessing import StandardScaler, MinMaxScaler
 from collections import defaultdict
 from ...wrapper.mysql import DerivedDatabaseConnector
 
-
 FUND_CLASSIFIER = {
-    'QDII': ['国际(QDII)股票型基金', '国际(QDII)债券型基金', '国际(QDII)另类投资基金', '国际(QDII)混合型基金'],
-    'stock': ['偏股混合型基金', '普通股票型基金', '平衡混合型基金', '灵活配置型基金'],  # 股票多空 对应绝对收益基金
-    'bond': ['混合债券型二级基金', '中长期纯债型基金', '混合债券型一级基金', '短期纯债型基金', '偏债混合型基金', '平衡混合型基金', '灵活配置型基金'],
+    'stock': ['偏股混合型基金', '普通股票型基金', '增强指数型基金', '被动指数型基金'],
+    'bond': ['混合债券型二级基金', '中长期纯债型基金', '增强指数型债券基金', '混合债券型一级基金', '被动指数型债券基金', '短期纯债型基金', '偏债混合型基金'],
+    'QDII': ['国际(QDII)另类投资基金', '国际(QDII)混合型基金', '国际(QDII)股票型基金', '国际(QDII)债券型基金', '股票多空', '商品型基金', 'REITs'],
     'mmf': ['货币市场型基金'],
     'mix': ['平衡混合型基金', '灵活配置型基金']
 }
 
+class DateJSONEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, datetime.date):
+            return obj.isoformat()
+        # Let the base class default method raise the TypeError
+        return json.JSONEncoder.default(self, obj)
+
 
 def normalize(a: Union[pd.DataFrame, pd.Series]) -> np.ndarray:
     if isinstance(a, pd.Series):
         a = pd.DataFrame(a)
     return StandardScaler().fit_transform(a)
 
-
 def score_rescale(a: Union[pd.DataFrame, pd.Series, np.ndarray]) -> np.ndarray:
-    if isinstance(a, pd.Series):
+    if isinstance(a, pd.Series) or isinstance(a, np.ndarray):
         a = pd.DataFrame(a)
+    a = a.rank()
     return MinMaxScaler().fit_transform(a) * 100
 
-
 def cont_rescale(a: Union[pd.DataFrame, pd.Series], feature_range: Tuple[float, float] = (0, 1)) -> np.ndarray:
     if isinstance(a, pd.Series):
         a = pd.DataFrame(a)
