@@ -23,8 +23,12 @@ from .barra_cne5_factor_processor import BarraCNE5FactorProcessor
 from .style_reg_processer import StyleBoxReg
 from .style_box import StyleBoxGenerator
 from .fund_manager_processor import ManagerProcessor
-from .fund_indicator_processor_ipo_conv import FundIndicatorProcessorIPOConv
 from .derived_top_manager_funds import TopManagerFundProcessor
+from .third_party_portfolio.qieman import QiemanPortfolioProcessor
+from .third_party_portfolio.danjuan import DanjuanPortfolioProcessor
+from .third_party_portfolio.em import EmPortfolioProcessor
+from .third_party_portfolio.gf_sec import GFSecPortfolioProcessor
+from .third_party_portfolio.manually_crawled import ManuallyCrawledPortfolioProcessor
 from ...api.basic import BasicDataApi
 
 
@@ -53,12 +57,16 @@ class DerivedDataProcessor:
         self.style_analysis_processors: List[StyleAnalysisProcessor] = []
         self.mng_fund_rank = FundManagerFundRankProcessor(self._data_helper)
         self._fund_alpha_processor = FundAlphaProcessor(self._data_helper)
-        self.fund_indicator_processor_ipo_conv = FundIndicatorProcessorIPOConv(self._data_helper)
         self.top_manager_funds_processor = TopManagerFundProcessor(self._data_helper)
         # 暂时只算这三个universe
         for universe in ('hs300', 'csi800', 'all'):
             sap = StyleAnalysisProcessor(self._data_helper, universe)
             self.style_analysis_processors.append(sap)
+        self._qieman_portfolio_processor = QiemanPortfolioProcessor(self._data_helper)
+        self._danjuan_portfolio_processor = DanjuanPortfolioProcessor(self._data_helper)
+        self._em_portfolio_processor = EmPortfolioProcessor(self._data_helper)
+        self._gf_sec_portfolio_processor = GFSecPortfolioProcessor(self._data_helper)
+        self._manually_crawled_portfolio_processor = ManuallyCrawledPortfolioProcessor(self._data_helper)
 
     def process_all(self, start_date, end_date):
         failed_tasks = []
@@ -73,10 +81,14 @@ class DerivedDataProcessor:
         # failed_tasks.extend(self._style_box_processor.process(start_date, end_date))
         failed_tasks.extend(self.mng_fund_rank.process(end_date))
         failed_tasks.extend(self._fund_alpha_processor.process(start_date, end_date))
-        failed_tasks.extend(self.fund_indicator_processor_ipo_conv.process(start_date, end_date))
         failed_tasks.extend(self.top_manager_funds_processor.process(start_date, end_date))
         for sap in self.style_analysis_processors:
             failed_tasks.extend(sap.process(start_date, end_date))
+        failed_tasks.extend(self._qieman_portfolio_processor.process(end_date))
+        failed_tasks.extend(self._danjuan_portfolio_processor.process(end_date))
+        failed_tasks.extend(self._em_portfolio_processor.process(end_date))
+        failed_tasks.extend(self._gf_sec_portfolio_processor.process(end_date))
+        failed_tasks.extend(self._manually_crawled_portfolio_processor.process(end_date))
 
         # 获取下一个交易日
         api = BasicDataApi()
