@@ -28,11 +28,12 @@ class FundScoreGroupProcess:
 
     def init(self, start_date: str, end_date: str):
         self._end_date = end_date
+        self._end_date_dt = pd.to_datetime(end_date).date()
         self._fund_info: pd.DataFrame = BasicDataApi().get_fund_info().drop(columns='_update_time').set_index('fund_id')
         # 获取计算评分需要用到的indicator
-        self._fund_indicator_group: pd.DataFrame = DerivedDataApi().get_fund_indicator_group(start_date, end_date, self._data_cycle).drop(columns='_update_time').set_index('fund_id')
+        self._fund_indicator_group_origin: pd.DataFrame = DerivedDataApi().get_fund_indicator_group(start_date, end_date, self._data_cycle).drop(columns='_update_time').set_index('fund_id')
         # 过滤掉scale为0的fund
-        self._fund_indicator_group = self._fund_indicator_group[self._fund_indicator_group.scale > 0]
+        self._fund_indicator_group = self._fund_indicator_group_origin[self._fund_indicator_group_origin.scale > 0]
         # 基金经理评分以及基金经理信息
         self._fund_manager_score: pd.DataFrame = DerivedDataApi().get_fund_manager_score(start_date=start_date, end_date=end_date).drop(columns='_update_time').set_index(['manager_id', 'fund_type'])
         self._fund_manager_info: pd.dataFrame = DerivedDataApi().get_fund_manager_info().drop(columns='_update_time').set_index('fund_id')
@@ -67,7 +68,7 @@ class FundScoreGroupProcess:
         if result1.shape[0] == 0:
             return
 
-        # 第一个维度打分
+        # 第一个维度打分 综合收益能力
         if wind_class == '普通股票型基金':
             ret_temp = 0.6 * normalize(data_to_calc_ret['sharpe']) + 0.2 * normalize(data_to_calc_ret['info_ratio_hs300']) + 0.2 * normalize(data_to_calc_ret['annual_ret'])
             result1['return_score'] = score_rescale(ret_temp * cont_rescale(data_to_calc_ret['continue_stats_v'], feature_range=self._FEATURE_RANGE))
@@ -94,10 +95,10 @@ class FundScoreGroupProcess:
 
         data_to_calc_others: pd.DataFrame = fin_data[fin_data.sharpe >= -20]
         result2 = pd.DataFrame(index=data_to_calc_others.index)
-        # 第二个维度打分
+        # 第二个维度打分 稳定性能力
         result2['robust_score'] = score_rescale(-normalize(data_to_calc_others[['annual_vol']]))
 
-        # 第三个维度打分
+        # 第三个维度打分 抗风险能力
         if wind_class in ('偏股混合型基金', '普通股票型基金', '平衡混合型基金', '灵活配置型基金', '增强指数型基金', '被动指数型基金', '股票多空'):
             risk_temp = normalize(-data_to_calc_others['mdd']) + normalize(-data_to_calc_others['mdd_len']) + normalize(data_to_calc_others['VaR']) + normalize(data_to_calc_others['CVaR'])
         elif wind_class in ('偏债混合型基金', '中长期纯债型基金', '短期纯债型基金', '混合债券型一级基金', '混合债券型二级基金', '增强指数型债券基金', '被动指数型债券基金'):
@@ -109,8 +110,9 @@ class FundScoreGroupProcess:
             risk_temp = normalize(-data_to_calc_others['mdd']) + normalize(data_to_calc_others['VaR'])
         result2['risk_score'] = score_rescale(risk_temp)
 
-        # 第四个和第五个维度计算
-        if wind_class in ('货币市场型基金', '被动指数型基金', '被动指数型债券基金'):
+        # 第四个和第五个维度计算 择时能力  选证能力
+        if wind_class in ('货币市场型基金', '被动指数型基金', '被动指数型债券基金', '中长期纯债型基金'):
+            data_to_calc_others['hold_num'] = data_to_calc_others['hold_num'].replace(0,1) # log 0 计算错误
             scale_temp = normalize(np.log(data_to_calc_others['scale'])) + normalize(np.log(data_to_calc_others['hold_num']))
             result2['timing_score'] = score_rescale(scale_temp)
             ins_temp = normalize(data_to_calc_others['ins_holds'])
@@ -120,22 +122,22 @@ class FundScoreGroupProcess:
             result2['selection_score'] = score_rescale(normalize(data_to_calc_others[['stock_cl_alpha']]))
 
         # 第六个维度 基金经理评分
-        wind_class_2 = self._fund_info.loc[fin_data.index, 'wind_class_2']
-        our_type = wind_class_2.map(self._wind_class_to_type)
-        managers = self._fund_manager_info.loc[self._fund_manager_info.index.intersection(fin_data.index), 'mng_id']
-        comp_key = pd.DataFrame(managers).join(our_type)
-        result3 = comp_key.join(self._fund_manager_score['score'], on=['mng_id', 'wind_class_2'], how='outer')
-        result3 = result3[result3.score.notna()]
-        # 在这里干掉重复的fund+基金经理的数据
-        result3 = result3.reset_index().drop_duplicates().set_index('fund_id')
+        mng_data = self._fund_info.loc[fin_data.index][['wind_class_2']]
+        mng_data.loc[:,'fund_type'] = mng_data.wind_class_2.map(self._wind_class_to_type)
+        managers_holds = self._fund_manager_info[( self._fund_manager_info.start_date <= self._end_date_dt)
+                                & (self._fund_manager_info.end_date >= self._end_date_dt)]
+        mng_data = mng_data.join(managers_holds[['mng_id']]).dropna(subset=['mng_id']).reset_index()
+        manager_score = self._fund_manager_score[['score']].reset_index().rename(columns={'manager_id':'mng_id'})
+        result3 = pd.merge(mng_data, manager_score, on = ['mng_id','fund_type']) 
         if not result3.empty:
-            result3 = result3['score'].groupby(by='fund_id', sort=False).mean()
-
+            result3 = result3[['score','fund_id']].groupby(by='fund_id').max().rename(columns={'score':'team_score'})
         # 合并所有的评分并计算总分
         total = result1.join(result2, how='outer')
         if not result3.empty:
-            total = total.join(result3.rename('team_score'), how='outer')
+            total = total.join(result3, how='outer')
         total['total_score'] = score_rescale(normalize(total.sum(axis=1).to_frame()))
+        if wind_class == '货币市场型基金':
+            total = total.loc[total['risk_score'].dropna().index]
         return total
 
     def calc(self):
@@ -150,7 +152,10 @@ class FundScoreGroupProcess:
             df2 = df2.reset_index().set_index('fund_id')
         # 两部分连到一起
         self._result = df1.append(df2)
-        self._result['datetime'] = self._end_date
+        self._result['team_score'] = self._result.team_score.fillna(0)
+        abnormal_df = self._fund_indicator_group_origin[~self._fund_indicator_group_origin['abnormal_reason'].isnull()]
+        self._result = self._result.join(abnormal_df[['abnormal_reason']],how='outer')
+        self._result['datetime'] = self._end_date_dt
         self._result['data_cycle'] = self._data_cycle
 
     def process(self, start_date: str, end_date: str, end_date_dt: datetime.date) -> List[str]:
