@@ -2,10 +2,12 @@ import datetime
 import pandas as pd
 import numpy as np
 import traceback
+from functools import partial
 from ...api.raw import RawDataApi
 from ...api.basic import BasicDataApi
 from ...view.derived_models import IndexValuationLongTerm
 from .derived_data_helper import DerivedDataHelper
+from ....util.calculator import *
 
 
 class IndexValProcessLongTerm:
@@ -73,9 +75,31 @@ class IndexValProcessLongTerm:
         self.result['val_score'] = score_list
         self.result = self.result.replace(np.inf, float('nan')).replace(-np.inf, float('nan'))
 
+        index_list = self.result.index_id.unique()
+        dic = {'pb_mrq':'pb_pct',
+               'pe_ttm':'pe_pct',
+               'roe':'roe_pct'}
+        pct_list = [0.3,0.7]
+        res = []
+        for index_id in index_list:
+            _df = self.result[self.result['index_id'] == index_id].set_index('datetime')
+            if _df.shape[0] < 242 *5:
+                continue
+            else:
+                for index_val, index_pct in dic.items():
+                    _res = []
+                    _df[index_val].dropna().rolling(window=10*242,min_periods=5*242).apply(
+                        partial(Calculator.roll_pct, pct_list=pct_list, res=_res), raw=True)
+                    _df_i = pd.DataFrame(_res, index=_df.index.tolist()[-len(_res):]).rename(columns={0:f'{index_pct}_30',1:f'{index_pct}_70'})
+                    _df = _df.join(_df_i)
+                res.append(_df.reset_index())
+        self.result = pd.concat(res, axis=0)
+
     def filter_est_peg(self, x):
+        if x is None:
+            return
         if x > 10000 or x < -10000:
-            return None
+            return
         return x
 
     def process(self, start_date, end_date):
@@ -91,6 +115,11 @@ class IndexValProcessLongTerm:
             self.result = self.result[self.result['datetime'] >= pd.to_datetime(start_date_dt)]
             self.result = self.result[self.result['datetime'] <= pd.to_datetime(end_date_dt)]
             self.result['est_peg'] = self.result['est_peg'].map(lambda x: self.filter_est_peg(x))
+
+            # 这里将sp500rmb的估值数据直接拷贝给sp500
+            sp500rmb_data = self.result.loc[self.result.index_id == 'sp500rmb', :].copy()
+            sp500rmb_data['index_id'] = 'sp500'
+            self.result = self.result.append(sp500rmb_data, ignore_index=False)
             self._data_helper._upload_derived(self.result, IndexValuationLongTerm.__table__.name)
         except Exception as e:
             print(e)
