@@ -7,10 +7,11 @@ import datetime
 import traceback
 import json
 import statsmodels.api as sm
+import time
 from scipy.optimize import Bounds, minimize
 from statsmodels.tsa.ar_model import AutoReg
 from pandas.tseries.offsets import DateOffset
-
+from ...api.raw import RawDataApi
 from ...api.basic import BasicDataApi
 from ...view.derived_models import FundIndicatorGroup
 from .derived_data_helper import DerivedDataHelper
@@ -23,6 +24,14 @@ class FundIndicatorProcessorGroup:
     _RISK_FEE_RATE = 0.011
     _RISK_FEE_RATE_PER_DAY = _RISK_FEE_RATE / _TRADING_DAYS_PER_YEAR
     _REPORT_DATE_LIST = ['0331', '0630', '0930', '1231']
+    _FUND_CLASSIFIER = {
+        'QDII': ['国际(QDII)股票型基金', '国际(QDII)债券型基金', '国际(QDII)另类投资基金', '国际(QDII)混合型基金', '股票多空', '商品型基金', 'REITs'],
+        'stock': ['偏股混合型基金', '普通股票型基金', '被动指数型基金', '增强指数型基金'], 
+        'bond': ['增强指数型债券基金','混合债券型二级基金', '中长期纯债型基金', '混合债券型一级基金', '短期纯债型基金', '偏债混合型基金', '被动指数型债券基金'],
+        'mmf': ['货币市场型基金'],
+        'mix': ['平衡混合型基金', '灵活配置型基金'],
+    }
+    _REPLACE_DICT = {'stock': 'csi_stockfund', 'bond': 'csi_boodfund', 'mmf': 'mmf', 'QDII': 'csi_f_qdii', 'mix': 'csi_f_hybrid'}
 
     def __init__(self, data_helper: DerivedDataHelper, data_cycle: int):
         self._data_helper = data_helper
@@ -33,11 +42,19 @@ class FundIndicatorProcessorGroup:
         self._rank_dic: Dict[pd.DataFrame] = {}
         self._res: List[pd.Series] = []
 
-    def init(self, end_date: str):
+    def init(self, end_date: str, is_print_time: bool=False):
         # self.log = []
+        self._wind_type_dict = {}
+        for type_i, type_list in self._FUND_CLASSIFIER.items():
+            for _ in type_list:
+                self._wind_type_dict.update({_:type_i})
+        # 获取待计算的基金列表
+        self.fund_info: pd.DataFrame = self._basic_api.get_fund_info().drop(columns='_update_time')
+        self._wind_class_2_dict = self.fund_info.set_index('fund_id')['wind_class_2'].to_dict()
+        self._fund_list: pd.DataFrame = self.fund_info[self.fund_info.structure_type <= 1]
+       
         # 向前取N年
         start_date: str = (pd.to_datetime(end_date, infer_datetime_format=True) - DateOffset(years=self._data_cycle+1)).strftime('%Y%m%d')
-        print(f'start date for data: {start_date}')
         # 获取区间内交易日列表
         all_trading_days: pd.Series = self._basic_api.get_trading_day_list().drop(columns='_update_time').set_index('datetime')
         self._trading_days: pd.Series = all_trading_days.loc[pd.to_datetime(start_date, infer_datetime_format=True).date():pd.to_datetime(end_date, infer_datetime_format=True).date()]
@@ -45,19 +62,45 @@ class FundIndicatorProcessorGroup:
         self.end_date = self._trading_days.index.array[-1]
 
         # 基金净值数据、指数价格数据的index对齐到交易日列表
-        fund_nav: pd.DataFrame = self._basic_api.get_fund_nav_with_date(start_date, end_date)  #, fund_list = f_l)
-        self._fund_nav: pd.DataFrame = fund_nav.pivot_table(index='datetime', columns='fund_id', values='adjusted_net_value').reindex(self._trading_days.index).fillna(method='ffill')
+        _tm_loads_nav_0 = time.time()
+        fund_nav: pd.DataFrame = self._basic_api.get_fund_nav_with_date(start_date, end_date, self._fund_list.fund_id.tolist())  #, fund_list = f_l)
+        self._fund_nav_origin: pd.DataFrame = fund_nav.pivot_table(index='datetime', columns='fund_id', values='adjusted_net_value').reindex(self._trading_days.index).fillna(method='ffill')
+        _tm_loads_nav_1 = time.time()
         # 选取成立满半年的 而且近半年波动率不为0，并排除掉异常净值基金
-        res = []
-        for fund_id in self._fund_nav:
-            _df = self._fund_nav[fund_id].dropna()
+        self.one_day_jump = self._fund_nav_origin.pct_change(1).abs().max()
+        self.three_day_jump = self._fund_nav_origin.pct_change(3).abs().max()
+        normal_fund_list = []
+        self.abnormal_reason = []
+        for fund_id in self._fund_nav_origin:
+            _df = self._fund_nav_origin[fund_id].dropna()
             if _df.shape[0] > 120:
                 if _df.iloc[-120:].var() > 1e-6:
-                    if _df.pct_change(1).abs().max() < 0.2:
-                        res.append(fund_id)
-        self._fund_nav = self._fund_nav[res].copy()
+                    wind_class_2 = self._wind_class_2_dict[fund_id]
+                    select_type = self._wind_type_dict.get(wind_class_2, None)
+                    if select_type is None:
+                        continue
+                    one_day_jump_i = self.one_day_jump[fund_id]
+                    three_day_jump_i = self.three_day_jump[fund_id]
+                    if select_type in ['stock','QDII']:
+                        if (one_day_jump_i < 0.1) & (three_day_jump_i < 0.25):
+                            normal_fund_list.append(fund_id)
+                        else:
+                            self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':f'high_risk: jump_max_1d {round(one_day_jump_i,3)} jump_max_3d {round(three_day_jump_i, 3)}'})
+                    
+                    elif select_type in ['bond','mix','mmf']:
+                        if (one_day_jump_i < 0.05) & (three_day_jump_i < 0.15):
+                            normal_fund_list.append(fund_id)
+                        else:
+                            self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':f'low_risk: jump_max_1d {round(one_day_jump_i,3)} jump_max_3d {round(three_day_jump_i, 3)}'})
+                else:
+                    self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':'price_inactive'})
 
+            else:
+                self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':'less_than_half_y'})
+        self._fund_nav = self._fund_nav_origin[normal_fund_list].copy()
+        _tm_filter_nav = time.time()
         index_price: pd.DataFrame = self._basic_api.get_index_price_dt(start_date, end_date, columns=['close'])
+        _tm_load_index = time.time()
         # 有的index最近没有数据，如活期存款利率/各种定期存款利率等，需要先reindex到全部的交易日列表上ffill
         self._index_price: pd.DataFrame = index_price.pivot_table(index='datetime', columns='index_id', values='close').reindex(self._trading_days.index).fillna(method='ffill')
         _bank_rate_df: pd.DataFrame = self._basic_api.get_index_price(index_list=['ddir', 'nonor', 'tmd_1y', 'tmd_2y', 'tmd_3m', 'tmd_3y', 'tmd_5y', 'tmd_6m', 'tmd_7d'])
@@ -65,9 +108,16 @@ class FundIndicatorProcessorGroup:
         drop_list = [i for i in self._index_price if i in _bank_rate_df]
         self._index_price = self._index_price.drop(drop_list, axis=1)
         self._index_price: pd.DataFrame = pd.concat([self._index_price, _bank_rate_df], axis=1)
+        
+        self.fund_type_benchmark = self._index_price[[self._REPLACE_DICT[fund_type] for fund_type in self._FUND_CLASSIFIER]]
+        self.fund_type_benchmark = self.fund_type_benchmark.rename(columns={v : f'{k}_bench' for k, v in self._REPLACE_DICT.items()})
         # 再reindex到参与计算的交易日列表上
         # self._index_price: pd.DataFrame = index_price.reindex(self._trading_days.index)
         pd.testing.assert_index_equal(self._fund_nav.index, self._index_price.index)
+        
+        macroeco = RawDataApi().get_em_macroeconomic_daily(codes=['EX_DR_RATIO']).drop(columns='_update_time')
+        self._macroeco: pd.DataFrame = macroeco.pivot_table(index='datetime', columns='codes', values='value').reindex(all_trading_days.index).fillna(method='ffill').reindex(self._trading_days.index)
+        pd.testing.assert_index_equal(self._fund_nav.index, self._macroeco.index)
         try:
             # 这个指数有一天的点数是0，特殊处理一下
             self._index_price['spi_spa'] = self._index_price['spi_spa'].where(self._index_price.spi_spa != 0).fillna(method='ffill')
@@ -84,16 +134,24 @@ class FundIndicatorProcessorGroup:
 
         # 计算benchmark return，且index对齐到fund_ret
         self._fund_benchmark_df: pd.DataFrame = self._basic_api.get_fund_benchmark().drop(columns='_update_time')
+        _tm_other_processor = time.time()
         benchmark_ret: pd.DataFrame = self._get_benchmark_return()
+        _tm_other_processor_tmp = time.time()
         self._benchmark_ret: pd.DataFrame = benchmark_ret.reindex(self._fund_ret.index)
+        benchmark_cols = self._benchmark_ret.columns.tolist()
+        for fund_id in self._fund_nav.columns:
+            benchmark_id = fund_id
+            if not benchmark_id in benchmark_cols:    
+                wind_class_2 = self._wind_class_2_dict[fund_id]
+                select_type = self._wind_type_dict[wind_class_2]
+                _benchmark_ret = self._index_price[self._REPLACE_DICT[select_type]].pct_change(1).rename(benchmark_id)
+                self._benchmark_ret = self._benchmark_ret.join(_benchmark_ret)
+
         pd.testing.assert_index_equal(self._fund_ret.index, self._benchmark_ret.index)
         self._fund_ret = self._fund_ret.join(self._benchmark_ret, rsuffix='_b')
-
-        # 获取待计算的基金列表
-        fund_list: pd.DataFrame = self._basic_api.get_fund_info().drop(columns='_update_time')
-        self._fund_list: pd.DataFrame = fund_list[fund_list.structure_type <= 1]
+        _tm_benchmakr_ret = time.time()
         # 获取wind一级分类
-        self._wind_class_1: np.ndarray = fund_list.wind_class_1.unique()
+        self._wind_class_1: np.ndarray = self._fund_list.wind_class_1.unique()
         # 按wind二级分类计算基金收益率排名
         self._get_continue_stats_tool()
         # 取出沪深300收益率
@@ -106,6 +164,7 @@ class FundIndicatorProcessorGroup:
         self._mmf = self._mmf.set_index('datetime')['close']
         self._mmf = self._mmf.apply(np.log).diff()[1:]
         self._fund_ret = self._fund_ret.join(self._mmf.rename('close_mmf'))
+        self._fund_ret = self._fund_ret.join(self.fund_type_benchmark.pct_change(1))
         # 获取年报/半年报的发布日期
         end_date_dt = pd.to_datetime(end_date, infer_datetime_format=True)
         md = end_date_dt.strftime('%m%d')
@@ -125,12 +184,30 @@ class FundIndicatorProcessorGroup:
         # 取出货基风险相关数据
         self._risk_metric = self._basic_api.get_fund_hold_asset_by_id(start_date=real_date, end_date=real_date)
         self._risk_metric = self._risk_metric[['fund_id', 'first_repo_to_nav', 'avg_ptm']].set_index('fund_id')
-
+        _tm_ret_process = time.time()
+        if is_print_time:
+            print(f'\t[time] fetch fund nav : {_tm_loads_nav_1 - _tm_loads_nav_0}')
+            print(f'\t[time] filter nav : {_tm_filter_nav - _tm_loads_nav_1}')
+            print(f'\t[time] fetch index price : {_tm_load_index - _tm_filter_nav}')
+            print(f'\t[time] fetch fund size and benchmark info : {_tm_other_processor - _tm_load_index}')
+            print(f'\t[time] calc fund benchmark ret : {_tm_other_processor_tmp - _tm_other_processor}')
+            print(f'\t[time] add fund benchmark ret : {_tm_benchmakr_ret - _tm_other_processor_tmp}')
+            print(f'\t[time] combine fund rets : {_tm_ret_process - _tm_benchmakr_ret}')
+            
     def history_load_data(self, end_date):
+        self._wind_type_dict = {}
+        for type_i, type_list in self._FUND_CLASSIFIER.items():
+            for _ in type_list:
+                self._wind_type_dict.update({_:type_i})
+        # 获取待计算的基金列表
+        self.fund_info: pd.DataFrame = self._basic_api.get_fund_info().drop(columns='_update_time')
+        self._wind_class_2_dict = self.fund_info.set_index('fund_id')['wind_class_2'].to_dict()
+        self._fund_list: pd.DataFrame = self.fund_info[self.fund_info.structure_type <= 1]
+       
         # 获取区间内交易日列表
         self._trading_days_all: pd.Series = self._basic_api.get_trading_day_list().drop(columns='_update_time').set_index('datetime')
         # 基金净值数据、指数价格数据的index对齐到交易日列表
-        self._fund_nav_all: pd.DataFrame = self._basic_api.get_fund_nav_with_date(end_date=end_date)
+        self._fund_nav_all: pd.DataFrame = self._basic_api.get_fund_nav_with_date(end_date=end_date,fund_list=self._fund_list.fund_id.tolist())
         self._fund_nav_all = self._fund_nav_all.pivot_table(index='datetime', columns='fund_id', values='adjusted_net_value')
         self._index_price_all: pd.DataFrame = self._basic_api.get_index_price_dt(end_date=end_date, columns=['close'])
         self._index_price_all = self._index_price_all.pivot_table(index='datetime', columns='index_id', values='close')
@@ -138,6 +215,8 @@ class FundIndicatorProcessorGroup:
         self._bank_rate_df = _bank_rate_df.pivot_table(index='datetime', columns='index_id', values='close').reindex(self._trading_days_all.index).fillna(method='ffill')
         drop_list = [i for i in self._index_price_all if i in self._bank_rate_df]
         self._index_price_all = self._index_price_all.drop(columns=drop_list)
+        self._macroeco_all = RawDataApi().get_em_macroeconomic_daily(codes=['EX_DR_RATIO']).drop(columns='_update_time')
+        self._macroeco_all = self._macroeco_all.pivot_table(index='datetime', columns='codes', values='value').reindex(self._trading_days_all.index).fillna(method='ffill')
         self._fund_size: pd.DataFrame = self._basic_api.get_fund_size_range(end_date=end_date)
         self._fund_size = self._fund_size.pivot_table(index='datetime', columns='fund_id', values='size')
         # 获取待计算的基金列表
@@ -163,20 +242,40 @@ class FundIndicatorProcessorGroup:
         _trading_days: pd.Series = self._trading_days_all.loc[start_date:end_date]
         # 使用实际trading day的最后一天作为end_date
         self.end_date = _trading_days.index.array[-1]
-
         # 基金净值数据、指数价格数据的index对齐到交易日列表
         fund_nav: pd.DataFrame = self._fund_nav_all.loc[start_date:end_date, :]
-        self._fund_nav: pd.DataFrame = fund_nav.reindex(_trading_days.index).fillna(method='ffill')
-        # 选取成立满半年的 而且近半年波动率不为0，并排除掉异常净值基金
-        res = []
-        for fund_id in self._fund_nav:
-            _df = self._fund_nav[fund_id].dropna()
+        self._fund_nav_origin: pd.DataFrame = fund_nav.reindex(_trading_days.index).fillna(method='ffill')
+        self.one_day_jump = self._fund_nav_origin.pct_change(1).abs().max()
+        self.three_day_jump = self._fund_nav_origin.pct_change(3).abs().max()
+        normal_fund_list = []
+        self.abnormal_reason = []
+        for fund_id in self._fund_nav_origin:
+            _df = self._fund_nav_origin[fund_id].dropna()
             if _df.shape[0] > 120:
                 if _df.iloc[-120:].var() > 1e-6:
-                    if _df.pct_change(1).abs().max() < 0.2:
-                        res.append(fund_id)
-        self._fund_nav = self._fund_nav[res].copy()
+                    wind_class_2 = self._wind_class_2_dict[fund_id]
+                    select_type = self._wind_type_dict.get(wind_class_2, None)
+                    if select_type is None:
+                        continue
+                    one_day_jump_i = self.one_day_jump[fund_id]
+                    three_day_jump_i = self.three_day_jump[fund_id]
+                    if select_type in ['stock','QDII']:
+                        if (one_day_jump_i < 0.1) & (three_day_jump_i < 0.25):
+                            normal_fund_list.append(fund_id)
+                        else:
+                            self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':f'high_risk: jump_max_1d {round(one_day_jump_i,3)} jump_max_3d {round(three_day_jump_i, 3)}'})
+                    
+                    elif select_type in ['bond','mix','mmf']:
+                        if (one_day_jump_i < 0.05) & (three_day_jump_i < 0.15):
+                            normal_fund_list.append(fund_id)
+                        else:
+                            self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':f'low_risk: jump_max_1d {round(one_day_jump_i,3)} jump_max_3d {round(three_day_jump_i, 3)}'})
+                else:
+                    self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':'price_inactive'})
 
+            else:
+                self.abnormal_reason.append({'fund_id':fund_id,'abnormal_reason':'less_than_half_y'})
+        self._fund_nav = self._fund_nav_origin[normal_fund_list].copy()
         index_price: pd.DataFrame = self._index_price_all.loc[start_date:end_date, :]
         # 有的index最近没有数据，如活期存款利率/各种定期存款利率等，需要先reindex到全部的交易日列表上ffill
         self._index_price: pd.DataFrame = index_price.reindex(_trading_days.index).fillna(method='ffill')
@@ -185,6 +284,9 @@ class FundIndicatorProcessorGroup:
         # 再reindex到参与计算的交易日列表上
         # self._index_price: pd.DataFrame = index_price.reindex(_trading_days.index)
         pd.testing.assert_index_equal(self._fund_nav.index, self._index_price.index)
+        
+        self._macroeco: pd.DataFrame = self._macroeco_all.reindex(_trading_days.index)
+        pd.testing.assert_index_equal(self._fund_nav.index, self._macroeco.index)
         try:
             # 这个指数有一天的点数是0，特殊处理一下
             self._index_price['spi_spa'] = self._index_price['spi_spa'].where(self._index_price.spi_spa != 0).fillna(method='ffill')
@@ -200,6 +302,14 @@ class FundIndicatorProcessorGroup:
         # 计算benchmark return，且index对齐到fund_ret
         self._benchmark_ret: pd.DataFrame = self._get_benchmark_return()
         self._benchmark_ret = self._benchmark_ret.reindex(self._fund_ret.index).dropna(how='all', axis=1)
+        benchmark_cols = self._benchmark_ret.columns.tolist()
+        for fund_id in self._fund_nav.columns:
+            benchmark_id = fund_id
+            if not benchmark_id in benchmark_cols:    
+                wind_class_2 = self._wind_class_2_dict[fund_id]
+                select_type = self._wind_type_dict[wind_class_2]
+                _benchmark_ret = self._index_price[self._REPLACE_DICT[select_type]].pct_change(1).rename(benchmark_id)
+                self._benchmark_ret = self._benchmark_ret.join(_benchmark_ret)
         pd.testing.assert_index_equal(self._fund_ret.index, self._benchmark_ret.index)
         self._fund_ret = self._fund_ret.join(self._benchmark_ret, rsuffix='_b')
         # 按wind二级分类计算基金收益率排名
@@ -208,6 +318,10 @@ class FundIndicatorProcessorGroup:
         self._fund_ret = self._fund_ret.join(self._hs300.rename('close_hs300'))
         self._fund_ret = self._fund_ret.join(self._mmf.rename('close_mmf'))
         self._fund_ret = self._fund_ret.dropna(how='all', axis=1)
+        
+        self.fund_type_benchmark = self._index_price[[self._REPLACE_DICT[fund_type] for fund_type in self._FUND_CLASSIFIER]]
+        self.fund_type_benchmark = self.fund_type_benchmark.rename(columns={v : f'{k}_bench' for k, v in self._REPLACE_DICT.items()})
+        self._fund_ret = self._fund_ret.join(self.fund_type_benchmark.pct_change(1))
         # 获取年报/半年报的发布日期
         md = end_date.strftime('%m%d')
         sentry_date = pd.Series(self._REPORT_DATE_LIST)
@@ -354,7 +468,11 @@ class FundIndicatorProcessorGroup:
                             break
                         else:
                             try:
-                                ra: pd.Series = self._index_price.loc[:, index]
+                                if index == 'nonor':
+                                    # 在这里我们用超额存款准备金率替代同业存款利率
+                                    ra: pd.Series = self._macroeco.loc[:, 'EX_DR_RATIO']
+                                else:
+                                    ra: pd.Series = self._index_price.loc[:, index]
                             except KeyError:
                                 # 表示我们没有该指数的价格数据
                                 # print(f'[benchmark_return] Error: Data Missing: {row.fund_id} {index} {index_raw}')
@@ -459,6 +577,12 @@ class FundIndicatorProcessorGroup:
         df_i_mmf = df_i_mmf[(~np.isnan(df_i_mmf)).all(axis=1)]
         df_i_mmf = df_i_mmf[-self._data_cycle_days:]
 
+        wind_class_2 = self._wind_class_2_dict[fund_id]
+        fund_type = self._wind_type_dict[wind_class_2]
+        df_i_cl = self._fund_ret[[fund_id]].join(self._fund_ret[[f'{fund_type}_bench']]).to_numpy()
+        df_i_cl = df_i_cl[(~np.isnan(df_i_cl)).all(axis=1)]
+        df_i_cl = df_i_cl[-self._data_cycle_days:]
+
         # alpha, beta
         alpha, beta = self._lambda_alpha_beta(df_i)
         # alpha, beta to the market
@@ -523,7 +647,7 @@ class FundIndicatorProcessorGroup:
             leverage = np.nan
             ptm = np.nan
 
-        stock_tm_res = self._lambda_tm(df_i)
+        stock_tm_res = self._lambda_tm(df_i_cl)
         stock_tm_alpha = stock_tm_res['alpha']
         stock_tm_alpha_t = stock_tm_res['alpha_t']
         stock_tm_alpha_p = stock_tm_res['alpha_p']
@@ -531,7 +655,7 @@ class FundIndicatorProcessorGroup:
         stock_tm_beta_t = stock_tm_res['beta_t']
         stock_tm_beta_p = stock_tm_res['beta_p']
 
-        stock_hm_res = self._lambda_hm(df_i)
+        stock_hm_res = self._lambda_hm(df_i_cl)
         stock_hm_alpha = stock_hm_res['alpha']
         stock_hm_alpha_t = stock_hm_res['alpha_t']
         stock_hm_alpha_p = stock_hm_res['alpha_p']
@@ -539,7 +663,7 @@ class FundIndicatorProcessorGroup:
         stock_hm_beta_t = stock_hm_res['beta_t']
         stock_hm_beta_p = stock_hm_res['beta_p']
 
-        stock_cl_res = self._lambda_cl(df_i)
+        stock_cl_res = self._lambda_cl(df_i_cl)
         stock_cl_alpha = stock_cl_res['alpha']
         stock_cl_alpha_t = stock_cl_res['alpha_t']
         stock_cl_alpha_p = stock_cl_res['alpha_p']
@@ -602,8 +726,14 @@ class FundIndicatorProcessorGroup:
 
     def calculate(self):
         fund_list = self._fund_list[['fund_id', 'wind_class_2']]
-        # 对fund_nav, benchmark_ret, fund_list三者交集的每个fund_id进行计算
-        self._result = fund_list[fund_list.fund_id.isin(self._fund_nav.columns.intersection(self._benchmark_ret.columns).to_list())].apply(lambda x: self._calculate_item(x.fund_id, x.wind_class_2), axis=1)
+        # 对fund_nav, fund_list二者交集的每个fund_id进行计算
+        self._result = fund_list[fund_list.fund_id.isin(self._fund_nav.columns.to_list())].apply(lambda x: self._calculate_item(x.fund_id, x.wind_class_2), axis=1)
+        self._result['ins_holds'] = self._result['ins_holds'].fillna(0)
+        self._result['hold_num'] = self._result['hold_num'].fillna(0)
+        # add abnormal
+        _df = pd.DataFrame(self.abnormal_reason)
+        _df['datetime'] = self.end_date
+        self._result = pd.merge(left=self._result, right=_df, how='outer', on=['fund_id','datetime'])
         self._result['data_cycle'] = str(self._data_cycle) + 'Y'
 
     def process(self, start_date: str, end_date: str, end_date_dt: datetime.date) -> List[str]:
