@@ -7,6 +7,7 @@ from functools import partial
 from sklearn.metrics import r2_score
 import concurrent.futures
 from multiprocessing import Pool
+from ....data.struct import AssetTimeSpan
 from ...manager.manager_fund import FundDataManager
 from ...manager.score import FundScoreManager
 from ...wrapper.mysql import DerivedDatabaseConnector
@@ -39,7 +40,7 @@ class FundIndicatorProcessor(object):
         self.fund_info = self._dm.dts.fund_info.copy()
         self.index_price = self._dm.dts.index_price.copy()
         
-        # 指数日线价格和基金净值 时间轴对齐 ，避免join后因为日期不存在出现价格空值
+        # 指数日线价格和基金净值 时间轴对齐 ，避免join后因为日期不存在出现价格空值_rolling_alpha_beta_time_ret_r2_base
         # 指数数据历史上出现非交易日有数据，join后，基金净值出现空值，长周期时间序列算指标出空值 
         # 避免对日收益fillna(0)， 造成track error 计算不符合实际
         # 计算逻辑空值填充情况： 1. 只对基金费率空值fillna(0) 
@@ -70,8 +71,9 @@ class FundIndicatorProcessor(object):
         # self.fund_ret = self.fund_ret.pivot_table(index = ['index_id','datetime'],columns='fund_id',values='ret')
         self.index_ret = np.log(self.index_price / self.index_price.shift(1))
         self.fund_to_index_dict = {fund_id:index_id for fund_id,index_id in self.fund_to_index_dict.items() if fund_id in self.fund_ret.columns}
-        self.index_list = self.fund_info.index_id.dropna().unique().tolist()
+        self.index_list = list(AssetTimeSpan.__dataclass_fields__.keys())
         self.index_fund = { index_id : [fund_idx for fund_idx, index_idx in self.fund_to_index_dict.items() if index_idx == index_id] for index_id in self.index_list}
+        self.index_fund['conv_bond'] = self._dm.dts.fund_conv_list
         self.start_date_dic = self.fund_info[['fund_id','start_date']].set_index('fund_id').to_dict()['start_date']
 
     def get_time_range(self, index_id):
@@ -88,12 +90,12 @@ class FundIndicatorProcessor(object):
     3. np.polyfit 一元回归，自带回归常数项，可以设置回归最高幂
     '''
 
-    def _rolling_alpha_beta_time_ret_r2(self, x, res, df):
+    def _rolling_alpha_beta_time_ret_r2(self, x, res, df, is_only_alpha_beta=False):
         # 回归相关的都在这里
         df_i = df.loc[x[0]:x[-1],]
-        return self._rolling_alpha_beta_time_ret_r2_base(res,df_i)
+        return self._rolling_alpha_beta_time_ret_r2_base(res,df_i,is_only_alpha_beta)
 
-    def _rolling_alpha_beta_time_ret_r2_base(self, res, df_i):
+    def _rolling_alpha_beta_time_ret_r2_base(self, res, df_i, is_only_alpha_beta=False):
         if sum(df_i.fund) == 0:
             res.append({'alpha':np.Inf,'beta':np.Inf,'time_ret':np.Inf})
             return 1
@@ -103,6 +105,9 @@ class FundIndicatorProcessor(object):
             r2 = r2_score(df_i.fund, p(df_i.benchmark))
             beta = ploy_res[0]
             alpha = ploy_res[1] * self.TRADING_DAYS_PER_YEAR
+            if is_only_alpha_beta:
+                res.append({'alpha':alpha,'beta':beta})
+                return 1
             day_len = df_i.shape[0]
             bar_num = int( day_len / self.MIN_TIME_SPAN)
 
@@ -147,6 +152,14 @@ class FundIndicatorProcessor(object):
         fund_indicator_part1 = res[0]
         if fund_indicator_part1['alpha'] in [np.Inf, -np.Inf]:
             return None
+        df_hs300 = self.fund_ret[[fund_id]].join(self.index_ret['hs300']).rename(columns={'hs300':'benchmark',fund_id:'fund'}).reset_index().iloc[-time_range:].dropna()
+        df_bond = self.fund_ret[[fund_id]].join(self.index_ret['national_debt']).rename(columns={'national_debt':'benchmark',fund_id:'fund'}).reset_index().iloc[-time_range:].dropna()
+        res = []
+        self._rolling_alpha_beta_time_ret_r2_base(res, df_hs300, is_only_alpha_beta=True)
+        fund_indicator_hs300 = res[0]
+        res = []
+        self._rolling_alpha_beta_time_ret_r2_base(res, df_bond, is_only_alpha_beta=True)
+        fund_indicator_bond = res[0]
         track_err = (df.fund - df.benchmark).std(ddof=1) * np.sqrt(self.TRADING_DAYS_PER_YEAR)
         timespan = int(time_range / self.TRADING_DAYS_PER_YEAR)
         fee_rate = self.fund_fee[fund_id]
@@ -188,57 +201,85 @@ class FundIndicatorProcessor(object):
             'm_square':m_square,
             'var':var,
             'sharpe':sharpe,
-            'year_length':year_length}
+            'year_length':year_length,
+            'alpha_bond':fund_indicator_bond['alpha'],
+            'beta_bond':fund_indicator_bond['beta'],
+            'alpha_hs300':fund_indicator_hs300['alpha'],
+            'beta_hs300':fund_indicator_hs300['beta'],}
         return dic
 
     def _process_fund_indicator_one(self, time_range, fund_ret, index_id, index_ret, fund_id):
-        df = fund_ret[[fund_id]].join(index_ret).dropna()
-        df = df.rename(columns={index_id:'benchmark',fund_id:'fund'}).reset_index()
-        df['year_length'] = df['datetime'].map(lambda x: (x-self.start_date_dic[fund_id]).days / self.NATURAL_DAYS_PER_YEAR)
-        res = []
-        pd.Series(df.index).rolling(
-            window=time_range,min_periods=self.MIN_TIME_SPAN).apply(
-                partial(self._rolling_alpha_beta_time_ret_r2, res=res, df=df), raw=True)
-        df = df.set_index('datetime')
-        df = df.join(pd.DataFrame(res,index=df.index[-len(res):]))
-        if 'alpha' not in df.columns:
-            return None
-        df['track_err'] = (df.fund - df.benchmark).rolling(window=time_range, min_periods=self.MIN_TIME_SPAN).std(ddof=1)* np.sqrt(self.TRADING_DAYS_PER_YEAR)
-        df['fund_id'] = fund_id
-        df['timespan'] = int(time_range / self.TRADING_DAYS_PER_YEAR)
-        df['fee_rate'] = self.fund_fee[fund_id]
-        df['info_ratio'] = df.alpha / df.track_err
-        df['mean_ret'] = df[['fund']].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean()
-        df['mean_ret_no_free_ret'] = df['mean_ret'] - self.RISK_FEE_RATE_PER_DAY
-        df['treynor'] = df['mean_ret_no_free_ret'] * time_range / df.beta
-        df['mdd'] = self.fund_nav[[fund_id]].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).apply(self._rolling_mdd, raw=True)
-        df['down_risk'] = np.abs(np.minimum(df['fund'] - self.RISK_FEE_RATE_PER_DAY, 0))
-        df['down_risk'] = df['down_risk'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean()* np.sqrt(self.TRADING_DAYS_PER_YEAR)
-        df['ret_over_period'] = self.fund_nav[[fund_id]] / self.fund_nav[[fund_id]].fillna(method='bfill').shift(time_range) - 1
-        df['annual_avg_daily_ret'] = df['fund'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean() * self.TRADING_DAYS_PER_YEAR 
-        df['annual_ret'] = self.fund_nav[[fund_id]].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).apply(self._rolling_annual_ret, raw=True)
-        df['annual_vol'] = df['fund'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).std(ddof=1) * np.sqrt(self.TRADING_DAYS_PER_YEAR)
-        df['vol_benchmark'] = df['benchmark'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).std(ddof=1) * np.sqrt(self.TRADING_DAYS_PER_YEAR)
-        df['mean_ret_benchmark'] = df[['benchmark']].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean()
-        df['m_square'] = df['vol_benchmark'] / df['annual_vol'] * df['mean_ret_no_free_ret'] + self.RISK_FEE_RATE_PER_DAY - df['mean_ret_benchmark']
-        df['var'] = df['fund'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).quantile(0.05)
-        df['var'] = np.minimum(df['var'], 0) * -1
-        df['sharpe'] = (df.annual_ret - self.RISK_FEE_RATE) / df.annual_vol
-        df = df.drop(['fund','benchmark','vol_benchmark','mean_ret_no_free_ret','mean_ret_benchmark','mean_ret'], axis=1)
-        # build history insert
-        df = df.replace({-np.Inf:None,np.Inf:None}).dropna(subset=['beta', 'alpha','track_err']).reset_index()
-        #self._data_helper._upload_derived(df, FundIndicatorDev.__table__.name)
+        try:
+            df = fund_ret[[fund_id]].join(index_ret).dropna()
+            df = df.rename(columns={index_id:'benchmark',fund_id:'fund'}).reset_index()
+            df['year_length'] = df['datetime'].map(lambda x: (x-self.start_date_dic[fund_id]).days / self.NATURAL_DAYS_PER_YEAR)
+            res = []
+            pd.Series(df.index).rolling(
+                window=time_range,min_periods=self.MIN_TIME_SPAN).apply(
+                    partial(self._rolling_alpha_beta_time_ret_r2, res=res, df=df), raw=True)
+            df = df.set_index('datetime')
+            df = df.join(pd.DataFrame(res,index=df.index[-len(res):]))
+            if 'alpha' not in df.columns:
+                return None
+            df_hs300 = self.fund_ret[[fund_id]].join(self.index_ret['hs300']).rename(columns={'hs300':'benchmark',fund_id:'fund'}).reset_index().dropna()
+            df_bond = self.fund_ret[[fund_id]].join(self.index_ret['national_debt']).rename(columns={'national_debt':'benchmark',fund_id:'fund'}).reset_index().dropna()
+            res_hs300 = []
+            pd.Series(df_hs300.index).rolling(
+                window=time_range,min_periods=self.MIN_TIME_SPAN).apply(
+                    partial(self._rolling_alpha_beta_time_ret_r2, res=res_hs300, df=df_hs300, is_only_alpha_beta=True), raw=True)
+            res_hs300 = pd.DataFrame(res_hs300, index=df_hs300.datetime[-len(res_hs300):]).rename(columns={'alpha':'alpha_hs300','beta':'beta_hs300'})
+            res_bond = []
+            pd.Series(df_bond.index).rolling(
+                window=time_range,min_periods=self.MIN_TIME_SPAN).apply(
+                    partial(self._rolling_alpha_beta_time_ret_r2, res=res_bond, df=df_bond, is_only_alpha_beta=True), raw=True)
+            res_bond = pd.DataFrame(res_bond, index=df_bond.datetime[-len(res_bond):]).rename(columns={'alpha':'alpha_bond','beta':'beta_bond'})
+            df = df.join(res_hs300).join(res_bond)
+            df['track_err'] = (df.fund - df.benchmark).rolling(window=time_range, min_periods=self.MIN_TIME_SPAN).std(ddof=1)* np.sqrt(self.TRADING_DAYS_PER_YEAR)
+            df['fund_id'] = fund_id
+            df['timespan'] = int(time_range / self.TRADING_DAYS_PER_YEAR)
+            df['fee_rate'] = self.fund_fee[fund_id]
+            df['info_ratio'] = df.alpha / df.track_err
+            df['mean_ret'] = df[['fund']].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean()
+            df['mean_ret_no_free_ret'] = df['mean_ret'] - self.RISK_FEE_RATE_PER_DAY
+            df['treynor'] = df['mean_ret_no_free_ret'] * time_range / df.beta
+            df['mdd'] = self.fund_nav[[fund_id]].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).apply(self._rolling_mdd, raw=True)
+            df['down_risk'] = np.abs(np.minimum(df['fund'] - self.RISK_FEE_RATE_PER_DAY, 0))
+            df['down_risk'] = df['down_risk'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean()* np.sqrt(self.TRADING_DAYS_PER_YEAR)
+            df['ret_over_period'] = self.fund_nav[[fund_id]] / self.fund_nav[[fund_id]].fillna(method='bfill').shift(time_range) - 1
+            df['annual_avg_daily_ret'] = df['fund'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean() * self.TRADING_DAYS_PER_YEAR 
+            df['annual_ret'] = self.fund_nav[[fund_id]].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).apply(self._rolling_annual_ret, raw=True)
+            df['annual_vol'] = df['fund'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).std(ddof=1) * np.sqrt(self.TRADING_DAYS_PER_YEAR)
+            df['vol_benchmark'] = df['benchmark'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).std(ddof=1) * np.sqrt(self.TRADING_DAYS_PER_YEAR)
+            df['mean_ret_benchmark'] = df[['benchmark']].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).mean()
+            df['m_square'] = df['vol_benchmark'] / df['annual_vol'] * df['mean_ret_no_free_ret'] + self.RISK_FEE_RATE_PER_DAY - df['mean_ret_benchmark']
+            df['var'] = df['fund'].rolling(window=time_range,min_periods=self.MIN_TIME_SPAN).quantile(0.05)
+            df['var'] = np.minimum(df['var'], 0) * -1
+            df['sharpe'] = (df.annual_ret - self.RISK_FEE_RATE) / df.annual_vol
+            df = df.drop(['fund','benchmark','vol_benchmark','mean_ret_no_free_ret','mean_ret_benchmark','mean_ret'], axis=1)
+            # build history insert
+            df = df.replace({-np.Inf:None,np.Inf:None}).dropna(subset=['beta', 'alpha','track_err']).reset_index()
+            #self._data_helper._upload_derived(df, FundIndicator.__table__.name)
+        except:
+            print(f'fund_id {fund_id} calc indicator error')
         return df
 
     def calculate_history(self, index_id_list:str=None, cpu_num:int=2):
         self.fund_fee = self.fund_info[['fund_id','manage_fee','trustee_fee']].set_index('fund_id').fillna(0).sum(axis = 1)
         result = []
         index_list = self.index_list if index_id_list is None else index_id_list
+        fund_index_dict = {}
         for index_id in index_list:
             time_range = self.get_time_range(index_id)
-            fund_list_1 = self.index_fund[index_id]
-            fund_list_2 = self._dm.dts.active_fund_info[self._dm.dts.active_fund_info['index_id'] == index_id].fund_id.tolist()
-            fund_list = list(set(fund_list_1).union(set(fund_list_2)).intersection(self._dm.dts.all_fund_list))
+            if index_id in ['active']:
+                continue
+            if index_id in ['conv_bond']:
+                fund_list = self.index_fund[index_id]
+                fund_list = [i for i in fund_list if i not in fund_index_dict]
+                fund_list = [i for i in fund_list if i in self.fund_ret.columns]
+            else:
+                fund_list_1 = self.index_fund[index_id]
+                fund_list_2 = self._dm.dts.active_fund_info[self._dm.dts.active_fund_info['index_id'] == index_id].fund_id.tolist()
+                fund_list = list(set(fund_list_1).union(set(fund_list_2)).intersection(self._dm.dts.all_fund_list))
             fund_ret = self.fund_ret[fund_list].copy()
             index_ret = self.index_ret[[index_id]]
             p = Pool()
@@ -247,6 +288,8 @@ class FundIndicatorProcessor(object):
             p.join()
             result.extend(res)    
             print(f'index {index_id} fund number : {len(fund_list)} finish' )
+            for fund_id in fund_list:
+                fund_index_dict[fund_id] = index_id
         result = [ _ for _ in result if not _ is None]
         self.result = pd.concat(result, axis=0, sort=True)
 
@@ -254,16 +297,26 @@ class FundIndicatorProcessor(object):
         self.fund_fee = self.fund_info[['fund_id','manage_fee','trustee_fee']].set_index('fund_id').fillna(0).sum(axis = 1)
         result = []
         index_list = self.index_list
+        fund_index_dict = {}
         for index_id in index_list:
             time_range = self.get_time_range(index_id)
-            fund_list_1 = self.index_fund[index_id]
-            fund_list_2 = self._dm.dts.active_fund_info[self._dm.dts.active_fund_info['index_id'] == index_id].fund_id.tolist()
-            fund_list = list(set(fund_list_1).union(set(fund_list_2)).intersection(self._dm.dts.all_fund_list))
+            if index_id in ['active']:
+                continue
+            if index_id in ['conv_bond']:
+                fund_list = self.index_fund[index_id]
+                fund_list = [i for i in fund_list if i not in fund_index_dict]
+                fund_list = [i for i in fund_list if i in self.fund_ret.columns]
+            else:
+                fund_list_1 = self.index_fund[index_id]
+                fund_list_2 = self._dm.dts.active_fund_info[self._dm.dts.active_fund_info['index_id'] == index_id].fund_id.tolist()
+                fund_list = list(set(fund_list_1).union(set(fund_list_2)).intersection(self._dm.dts.all_fund_list))
             fund_ret = self.fund_ret[fund_list].copy()
             index_ret = self.index_ret[[index_id]]
             for fund_id in fund_list:
                 result.append(self._process_fund_indicator_update(fund_id, time_range, fund_ret, index_id, index_ret))
             print(f'index {index_id} finish')
+            for fund_id in fund_list:
+                fund_index_dict[fund_id] = index_id
         result = [ _ for _ in result if not _ is None]
         self.result = pd.DataFrame(result)
         self.result = self.result.replace({-np.Inf:None,np.Inf:None}).dropna(subset=['beta', 'alpha','track_err'])
