@@ -1,5 +1,5 @@
 
-from typing import Dict, Set
+from typing import Dict
 
 import traceback
 import math
@@ -37,6 +37,9 @@ class FundIndicatorProcessorMonthly:
         # 过滤出来需要用到的fund_info
         self._fund_info: pd.DataFrame = self._dm.dts.fund_info[self._dm.dts.fund_info['fund_id'].isin(self._fund_nav.columns)]
 
+        # 获取fund到index的字典
+        fund_to_index: Dict[str, str] = self._fund_info[['fund_id', 'index_id']].set_index('fund_id').to_dict()['index_id']
+
         # 超过基金终止日的基金净值赋空
         fund_to_end_date_dict = self._fund_info[['fund_id', 'end_date']].set_index('fund_id').to_dict()['end_date']
         for fund_id in self._fund_nav.columns:
@@ -44,16 +47,19 @@ class FundIndicatorProcessorMonthly:
             if self.end_date >= fund_end_date:
                 self._fund_nav.loc[fund_end_date:, fund_id] = np.nan
 
-        # resample
         self._index_ret: pd.DataFrame = self._get_return(self._index_price)
         self._fund_ret: pd.DataFrame = self._get_return(self._fund_nav)
+        index_list: pd.Series = self._fund_ret.columns.map(fund_to_index)
+        index_list = index_list[index_list.notna()]
+        self._index_list: np.ndarray = index_list.unique()
+
+        # resample
         self._index_ret_resampled: pd.DataFrame = self._get_resample_ret(self._index_ret)
         # self._national_debt_ret_resampled: pd.DataFrame = self._index_ret_resampled.loc[:, 'national_debt']
         self._fund_ret_resampled: pd.DataFrame = self._get_resample_ret(self._fund_ret)
 
-        self._index_fund: pd.DataFrame = self._fund_info[['index_id', 'fund_id']].groupby('index_id').apply(lambda x: x['fund_id'].to_list())
+        self._index_fund: pd.DataFrame = self._fund_info.loc[self._fund_info.index_id.isin(self._index_list), ['index_id', 'fund_id']].groupby('index_id').apply(lambda x: x['fund_id'].to_list())
         self._fund_start_date: Dict[str] = self._fund_info[['fund_id', 'start_date']].set_index('fund_id').to_dict()['start_date']
-        self._index_set: Set[str] = set(self._fund_info.loc[:, 'index_id'].to_list())
 
     def _get_return(self, df) -> pd.DataFrame:
         ret = df.applymap(np.log).diff()
@@ -74,7 +80,7 @@ class FundIndicatorProcessorMonthly:
 
         # treynor ratio
         # ex_ret = fund_ret_resampled.sub(self._national_debt_ret_resampled, axis=0)
-        ex_ret = fund_ret_resampled.sub(self._RF, axis=0)
+        ex_ret = fund_ret_resampled.sub(self._RF / 12, axis=0)
         ex_ret_sum = ex_ret.sum()
         annualized_ret_ex = ex_ret_sum * 12 / len(fund_ret_resampled)
         treynor_ratio = annualized_ret_ex / beta
@@ -104,7 +110,7 @@ class FundIndicatorProcessorMonthly:
 
     def _calculate_update(self):
         result = []
-        for index_id in self._index_set:
+        for index_id in self._index_list:
             _tm_1 = time.time()
             fund_list = self._index_fund[index_id]
             fund_list = list(set(fund_list) & set(self._fund_ret.columns))
@@ -116,25 +122,19 @@ class FundIndicatorProcessorMonthly:
             _tm_2 = time.time()
             print(f'index {index_id} fund number : {len(fund_list)} finish, cost time {_tm_2 - _tm_1}s')
         self.result = pd.concat(result)
-        self.result = self.result.replace(np.Inf, None).replace(-np.Inf, None)
+        self.result = self.result.replace({-np.Inf:None,np.Inf:None})
 
-    def process(self, start_date, end_date, next_trading_day):
+    def process(self, start_date: str, end_date: str, end_date_dt: datetime.date):
         # 月更因子, 由于resample出来的日期都是每月最后一天，所以每月最后一天前的最后一个交易日更新
         failed_tasks = []
         try:
-            end_date_dt = pd.to_datetime(end_date, infer_datetime_format=True).date()
-            next_trading_day_dt = pd.to_datetime(next_trading_day, infer_datetime_format=True).date()
-            if end_date_dt.year == next_trading_day_dt.year and end_date_dt.month == next_trading_day_dt.month:
-                print(f'monthly indicator only update on the last day of month, not today {end_date_dt}')
-                return []
-            else:
-                print(f'monthly indicator update on the last day of month {end_date_dt}')
-                start_date_dt = datetime.datetime.strptime(start_date, '%Y%m%d').date()
-                start_date = start_date_dt - datetime.timedelta(days=1150)  # 3年历史保险起见，多取几天 3*365=1095
-                start_date = datetime.datetime.strftime(start_date, '%Y%m%d')
-                self.init(start_date=start_date, end_date=end_date)
-                self._calculate_update()
-                self._data_helper._upload_derived(self.result.assign(datetime=end_date_dt).reset_index(), FundIndicatorMonthly.__table__.name)
+            print(f'monthly indicator update on the last day of month {end_date_dt}')
+            start_date_dt = pd.to_datetime(start_date, infer_datetime_format=True).date()
+            start_date = start_date_dt - datetime.timedelta(days=1150)  # 3年历史保险起见，多取几天 3*365=1095
+            start_date = datetime.datetime.strftime(start_date, '%Y%m%d')
+            self.init(start_date=start_date, end_date=end_date)
+            self._calculate_update()
+            self._data_helper._upload_derived(self.result.assign(datetime=end_date_dt).reset_index(), FundIndicatorMonthly.__table__.name)
         except Exception as e:
             print(e)
             traceback.print_exc()
