@@ -7,7 +7,7 @@ from ...wrapper.mysql import (RawDatabaseConnector, BasicDatabaseConnector, Deri
 from ...view.view_models import FundDailyCollection
 from ...view.basic_models import FundNav, FundInfo, FundRate, FundSize, FundRet, FundBenchmark, IndexInfo, \
     FundStatusLatest
-from ...view.derived_models import FundAlpha, FundScore, FundIndicator, FundScoreNew
+from ...view.derived_models import FundAlpha, FundIndicator, FundScoreNew, FundManagerInfo
 from ....constant import ExchangeStatus
 
 class FundDailyCollectionProcessor(object):
@@ -22,7 +22,6 @@ class FundDailyCollectionProcessor(object):
                     FundInfo.start_date,
                     FundInfo.end_date,
                     # FundInfo.manager_id,
-                    FundInfo.fund_manager,
                     FundInfo.company_id,
                     FundInfo.benchmark,
                     FundInfo.desc_name,
@@ -191,6 +190,33 @@ class FundDailyCollectionProcessor(object):
             except Exception as e:
                 print('Failed fill get data <err_msg> {}'.format(e))
 
+    def get_fund_manager_info_df(self):
+        def helper(x):
+            x['mng_name'] = '、'.join(x['mng_name'])
+            x['mng_id'] = '、'.join(x['mng_id'])
+            return x
+
+        with DerivedDatabaseConnector().managed_session() as mn_session:
+            try:
+                query = mn_session.query(
+                    FundManagerInfo.fund_id,
+                    FundManagerInfo.mng_id,
+                    FundManagerInfo.mng_name,
+                ).filter(
+                    FundManagerInfo.end_date == '2040-01-01',
+                )
+                df = pd.read_sql(query.statement, query.session.bind)
+                df = df.groupby(by='fund_id').apply(helper)
+                df = df.drop_duplicates(subset=['fund_id'], keep='last')
+                df = df.rename(columns={
+                    'mng_name': 'fund_manager',
+                    'mng_id': 'fund_manager_id',
+                })
+                df = df.set_index('fund_id')
+                return df
+            except Exception as e:
+                print('Failed fill get data <err_msg> {}'.format(e))
+
     def get_exchange_status_df(self):
         with BasicDatabaseConnector().managed_session() as mn_session:
             try:
@@ -235,23 +261,6 @@ class FundDailyCollectionProcessor(object):
             except Exception as e:
                 print('Failed fill get data <err_msg> {}'.format(e))
 
-    def get_score_df(self):
-        with DerivedDatabaseConnector().managed_session() as mn_session:
-            try:
-                latest_time = mn_session.query(FundScore.datetime).order_by(FundScore.datetime.desc()).limit(1).one_or_none()[0]
-                print(latest_time)
-                query = mn_session.query(
-                    FundScore.fund_id,
-                    FundScore.tag_name,
-                    FundScore.score,
-                ).filter(FundScore.datetime==latest_time)
-                df = pd.read_sql(query.statement, query.session.bind)
-                df = df.drop_duplicates(subset=['fund_id'], keep='last')
-                return df
-
-            except Exception as e:
-                print('Failed get_score_df <err_msg> {}'.format(e))
-
     def get_score_new_df(self):
         with DerivedDatabaseConnector().managed_session() as mn_session:
             try:
@@ -269,6 +278,7 @@ class FundDailyCollectionProcessor(object):
                     FundScoreNew.selection_score,
                     FundScoreNew.team_score,
                     FundScoreNew.total_score,
+                    FundScoreNew.abnormal_reason,
                 ).filter(
                     FundScoreNew.fund_id == subq.c.temp_id,
                     FundScoreNew.datetime == subq.c.temp_date,
@@ -277,6 +287,7 @@ class FundDailyCollectionProcessor(object):
 
                 df = pd.read_sql(query.statement, query.session.bind)
                 df = df.drop_duplicates(subset=['fund_id'], keep='last')
+                df = df[df['abnormal_reason'].isnull()].drop(columns='abnormal_reason')
                 return df
 
             except Exception as e:
@@ -399,7 +410,9 @@ class FundDailyCollectionProcessor(object):
             size = self.get_size_df()
             print(size)
             print('-' * 100)
-            score = self.get_score_df().set_index('fund_id')
+            # TODO: need to remove this one
+            score = pd.DataFrame()
+            # score = self.get_score_df().set_index('fund_id')
             print(score)
             print('-' * 100)
             new_score = self.get_score_new_df().set_index('fund_id')
@@ -413,6 +426,11 @@ class FundDailyCollectionProcessor(object):
             print(exchange_status)
             print('-' * 100)
 
+            print('处理基金经理')
+            fund_manager = self.get_fund_manager_info_df()
+            print(fund_manager)
+
+
             print('处理机构评级')
             rating = self.get_institution_rating_df()
             rating = rating.drop_duplicates(subset=['fund_id'], keep='last')
@@ -421,7 +439,7 @@ class FundDailyCollectionProcessor(object):
             print('-' * 100)
 
             print('处理净值')
-            df = pd.concat([funds, bench, nav, alpha, size, rating, score, new_score, tag_indicator, exchange_status], axis=1, sort=False)
+            df = pd.concat([funds, bench, nav, alpha, size, rating, score, new_score, tag_indicator, exchange_status, fund_manager], axis=1, sort=False)
             df.index.name = 'fund_id'
             print('_'*100)
             print(df)
