@@ -33,13 +33,13 @@ class FundDataTables:
     fund_indicator_annual: pd.DataFrame = None
     _barra_cne5_factor_return: pd.DataFrame = None
     # fund helps
-    fund_ipo_list: set = None
     fund_conv_list: set = None
     fund_list: set = None
     all_fund_list: set = None
     index_list: set = None
     fund_index_map: dict = None  # fund_id -> index_id
     index_date_list: list = None
+    fund_end_date_dict: dict = None
     # score helps
     index_fund_indicator_pack: pd.DataFrame = None  # pivot_table
 
