@@ -2,7 +2,7 @@
 import os
 import dataclasses
 import time
-from typing import List
+from typing import List, Optional
 from collections import defaultdict
 
 import pandas as pd
@@ -13,7 +13,7 @@ from ...util.config import SurfingConfigurator, AWSSettings
 from ..wrapper.mysql import RawDatabaseConnector, BasicDatabaseConnector, DerivedDatabaseConnector
 from ..view.raw_models import CSIndexComponent
 from ..view.basic_models import FundInfo, TradingDayList, IndexInfo, IndexPrice, FundNav, Fund_size_and_hold_rate, FundBenchmark, FundIPOStats
-from ..view.basic_models import FundConvStats
+from ..view.basic_models import FundConvStats, FundStatusLatest, FundOpenInfo
 from ..view.derived_models import FundIndicator, FundIndicatorAnnual, IndexValuationLongTerm, FundIndicatorWeekly
 from ..view.derived_models import FundIndicatorMonthly, BarraCNE5FactorReturn, FundManagerScore, FundManagerInfo, FundManagerFundRank
 from .data_tables import FundDataTables
@@ -70,6 +70,8 @@ class SynchronizerS3ForDM(S3ETLTool):
         FundBenchmark,
         FundIPOStats,
         FundConvStats,
+        FundStatusLatest,
+        FundOpenInfo,
     ]
     # DM中需要使用的derived table
     _DERIVED_TABLE_LIST = [
@@ -275,22 +277,25 @@ class SynchronizerS3ForDM(S3ETLTool):
 class S3RetrieverForDM(S3ETLTool):
     '''从S3获取数据的工具'''
 
-    def __init__(self, activation: bool = False):
+    def __init__(self, activation: Optional[List[str]] = None):
         super().__init__()
         # 默认先不激活
-        self._activation: bool = activation
+        self._activation: S3RetrieverForDM = activation
 
     @property
     def is_activated(self) -> bool:
-        return self._activation
+        # 只有self._activation是空列表的情况才返回True; 非空表示一部分table需要从mysql读取
+        return self._activation is not None and not self._activation
 
     def fetch_table(self, query, post_processor=None):
         table_name = query.selectable.froms[0].name
         # 如果没有开启的话，直接从mysql读取
-        if not self._activation:
+        if self._activation is None or (table_name in self._activation):
+            print(f'loading data of {table_name} from sql')
             return pd.read_sql(query.statement, query.session.bind).drop(columns='_update_time', errors='ignore')
 
         try:
+            print(f'loading data of {table_name} from s3')
             df = pd.read_parquet(f'{self._raw_s3_bucket_uri}/{table_name}.parquet').drop(columns='_update_time', errors='ignore')
             columns_need = []
             col_desc = query.column_descriptions
@@ -314,7 +319,7 @@ class S3RetrieverForDM(S3ETLTool):
                 df['price_source'] = df.price_source.transform(lambda x: IndexPriceSource(x))
             return df
         except Exception as e:
-            print(f'WARNING!! fetch table {table_name} from s3 failed (err_msg){e}, try mysql')
+            print(f'WARNING!! fetch table {table_name} from s3 failed (err_msg){type(e).__name__}: {e}, try mysql')
             # TODO: 如果这里DB也失败，应该也需要适当的处理
             return pd.read_sql(query.statement, query.session.bind).drop(columns='_update_time', errors='ignore')
 
@@ -367,7 +372,7 @@ class S3RetrieverForDM(S3ETLTool):
                     data = set(df.to_dict(orient='list')[name])
                     if name == 'index_date_list':
                         data = sorted(data)
-                elif name in ('fund_index_map'):
+                elif name in ('fund_index_map', 'fund_end_date_dict'):
                     data = df.to_dict()[name]
                 else:
                     raise FileNotFoundError(f'do not support retrieve this data {name} from s3')
