@@ -35,6 +35,11 @@ BLACK_SHEEP.extend(['164210!1','164703!1', '000428!0'])
 ## 000022!0 南方中债中期票据A
 BLACK_SHEEP.extend(['000169!1','000022!0'])
 
+# 基金类型改变 货币型 变债券型 2020-10-22 稳健策略净值波动
+## 380010!0    中银理财30天a -> 中银聚享债券a
+## 380011!0    中银理财30天b -> 中银聚享债券b
+BLACK_SHEEP.extend(['380010!0','380011!0'])
+
 EHR_PROBLEM_EKYWORDS = {
     'E': ['ETF', 'REITs', 'ESG', 'CES'],
     'H': ['H股', 'A-H50', '50AH', '中证AH'],
@@ -200,14 +205,9 @@ def active_fund_info(fund_info: pd.DataFrame, fund_benchmark: pd.DataFrame):
     active_fund_info = active_fund_info[active_fund_info['start_date'] < _begin_date]
     return active_fund_info
 
-def get_ipos_funds(ipo_weight_df):
-    # 过去一年如果打新金额占比大于0.15， 作为打新型基金
-    ipo_weight_df = ipo_weight_df.pivot_table(index='end_date', columns='fund_id', values='ipo_allocation_weight').fillna(0)
-    ipo_weight_last_year_df = ipo_weight_df + ipo_weight_df.shift(1).fillna(0) + ipo_weight_df.shift(2).fillna(0) + ipo_weight_df.shift(3).fillna(0)
-    ipo_funds = ipo_weight_last_year_df.columns[ipo_weight_last_year_df.max() > 0.15].values.tolist()
-    return ipo_funds
-
-def get_conv_funds(fund_conv_stats):
+def get_conv_funds(fund_conv_stats, fund_info):
     conv_df = fund_conv_stats.pivot_table(index='datetime', values='conv_weights', columns='fund_id')
     conv_funds = conv_df.columns[conv_df.max() > 20].tolist()
+    conv_funds = fund_info[(fund_info['fund_id'].isin(conv_funds)) 
+              & (fund_info.structure_type == 0)].fund_id.tolist()
     return conv_funds
\ No newline at end of file
