@@ -10,8 +10,9 @@ import copy
 import json
 import contextlib
 import re
+from ...stock.factor.api import StockFactorApi
 from ..wrapper.mysql import RawDatabaseConnector, BasicDatabaseConnector, DerivedDatabaseConnector
-from .fund_info_filter import filter_fund_info, fund_info_update, active_fund_info, get_ipos_funds, get_conv_funds
+from .fund_info_filter import filter_fund_info, fund_info_update, active_fund_info, get_conv_funds
 from ..struct import AssetPrice, FundScoreParam, TaaTunerParam, AssetTimeSpan
 from .score import FundScoreManager, ScoreFunc
 from .data_tables import FundDataTables
@@ -20,7 +21,9 @@ from ..view.raw_models import CSIndexComponent
 from ..view.basic_models import *
 from ..view.derived_models import *
 
-INDEX_REPLACE = ('active','csi_stockfund')
+INDEX_REPLACE = {'active':'csi_stockfund',
+                 'conv_bond':'csi_convex'}
+INDEX_REPLACE_REVERSE = {v : k for k, v in INDEX_REPLACE.items()}
 
 @contextlib.contextmanager
 def profiled(file_name=None):
@@ -46,7 +49,7 @@ class FundDataManager(DataManager):
 
     _BLOCK_DF_LIST = ('fund_manager_score', 'fund_indicator')
 
-    def __init__(self, start_time=None, end_time=None, fund_score_param: FundScoreParam = None, score_manager: FundScoreManager = None, s3_retriever_activation: bool = False):
+    def __init__(self, start_time=None, end_time=None, fund_score_param: FundScoreParam = None, score_manager: FundScoreManager = None, s3_retriever_activation: Optional[List[str]] = None):
         DataManager.__init__(self,
             start_time or datetime.datetime(2005, 1, 1),
             end_time or datetime.datetime.now()
@@ -147,7 +150,7 @@ class FundDataManager(DataManager):
         print('could not init from s3 directly, do normal initialization')
         _tm_start = time.time()
         index_list = index_list or list(AssetTimeSpan.__dataclass_fields__.keys())
-        index_list = [_.replace(INDEX_REPLACE[0],INDEX_REPLACE[1]) for _ in index_list]
+        index_list = [INDEX_REPLACE.get(_, _) for _ in index_list]
 
         def fetch_table(session, view):
             query = session.query(view)
@@ -225,11 +228,11 @@ class FundDataManager(DataManager):
 
         def _fund_manager_score_filter(query):
             def _fund_manager_score_filter_etl(x):
-                return x[x.fund_type.isin(['stock']) & (x.datetime >= self.start_date) & (x.datetime <= self.end_date)].reset_index(drop=True)
+                return x[x.fund_type.isin(['stock']) & (x.datetime >= (self.start_date - datetime.timedelta(days=20))) & (x.datetime <= self.end_date)].reset_index(drop=True)
 
             return query.filter(
                 FundManagerScore.fund_type.in_(['stock']),
-                FundManagerScore.datetime >= self.start_date,
+                FundManagerScore.datetime >= self.start_date - datetime.timedelta(days=20),
                 FundManagerScore.datetime <= self.end_date,
             ), _fund_manager_score_filter_etl
 
@@ -258,6 +261,7 @@ class FundDataManager(DataManager):
             _tm_basic_fetch_info = time.time()
             self.dts.fund_info = fund_info_update(self.dts.fund_info)
             self.dts.fund_status_latest = fetch_table(quant_session, FundStatusLatest)
+            self.dts.fund_end_date_dict = self.dts.fund_info.set_index('fund_id').to_dict()['end_date']
             _fund_open_info = fetch_table(quant_session, FundOpenInfo)
             _filter_fund_info = filter_fund_info(self.dts.fund_info, index_list)
             _tm_basic_filter_info = time.time()
@@ -267,12 +271,10 @@ class FundDataManager(DataManager):
             close_fund_list = _fund_open_info.fund_id.tolist()
             self.dts.fund_ipo_stats = fetch_table(quant_session, FundIPOStats)
             self.dts.fund_conv_stats = fetch_table(quant_session, FundConvStats)
-            self.dts.fund_ipo_list = get_ipos_funds(self.dts.fund_ipo_stats)
-            self.dts.fund_conv_list = get_conv_funds(self.dts.fund_conv_stats)
-            self.dts.fund_ipo_list = {i for i in self.dts.fund_ipo_list if (i not in in_market_fund_list) and (i not in close_fund_list)}
+            self.dts.fund_conv_list = get_conv_funds(self.dts.fund_conv_stats, self.dts.fund_info)
             self.dts.fund_conv_list = {i for i in self.dts.fund_conv_list if (i not in in_market_fund_list) and (i not in close_fund_list)}
             self.dts.all_fund_list = {i for i in self.dts.all_fund_list if (i not in in_market_fund_list) and (i not in close_fund_list)}
-            self.dts.all_fund_list = self.dts.all_fund_list.union(self.dts.fund_ipo_list).union(self.dts.fund_conv_list)
+            self.dts.all_fund_list = self.dts.all_fund_list.union(self.dts.fund_conv_list)
             _normal_funds_index_dic = {cur.fund_id: cur.index_id for cur in _filter_fund_info.itertuples()}
             _active_funds_index_dic = {i.fund_id: i.index_id for i in self.dts.active_fund_info.itertuples()}
             self.dts.fund_index_map = {**_normal_funds_index_dic, **_active_funds_index_dic}
@@ -281,6 +283,10 @@ class FundDataManager(DataManager):
             self.dts.trading_days = fetch_table(quant_session, TradingDayList)
             self.dts.trading_days = self.dts.trading_days[self.dts.trading_days.datetime <= self.end_date]
             trim_trading_days = self.dts.trading_days[(self.dts.trading_days.datetime >= self.start_date) & (self.dts.trading_days.datetime <= self.end_date)].datetime
+            ipo_weight_last_year_df = self.dts.fund_ipo_stats.pivot_table(index='end_date', columns='fund_id', values='ipo_allocation_weight').fillna(0)
+            _trim_trading_days = ipo_weight_last_year_df.index.union(trim_trading_days)
+            ipo_weight_last_year_df = ipo_weight_last_year_df + ipo_weight_last_year_df.shift(1).fillna(0) + ipo_weight_last_year_df.shift(2).fillna(0) + ipo_weight_last_year_df.shift(3).fillna(0)
+            ipo_weight_last_year_df = ipo_weight_last_year_df.reindex(_trim_trading_days).fillna(method='ffill').stack().reset_index().rename(columns={0:'ipo_weight','level_0':'datetime'})
             _tm_basic_fetch_days = time.time()
             # index
             self.dts.index_info = fetch_table(quant_session, IndexInfo)
@@ -292,7 +298,7 @@ class FundDataManager(DataManager):
             )
             _index_query, _index_price_filter_func = _index_price_filter(_index_query, index_list)
             index_price_raw = self._s3_retriever.fetch_table(_index_query, _index_price_filter_func)
-            self.dts.index_price = index_price_raw.pivot_table(index='datetime', columns='index_id', values='close').reindex(trim_trading_days).fillna(method='ffill').rename(columns = {INDEX_REPLACE[1]:INDEX_REPLACE[0]})
+            self.dts.index_price = index_price_raw.pivot_table(index='datetime', columns='index_id', values='close').reindex(trim_trading_days).fillna(method='ffill').rename(columns = INDEX_REPLACE_REVERSE)
             _tm_basic_fetch_index_price = time.time()
             # fund nav
             _fund_nav_query = quant_session.query(
@@ -327,8 +333,10 @@ class FundDataManager(DataManager):
             )
             _fund_size_and_hold_rate_query, _fund_size_and_hold_rate_filter_func = _fund_size_and_hold_rate_filter(_fund_size_and_hold_rate_query,self.dts.all_fund_list)
             _size_and_hold_rate = self._s3_retriever.fetch_table(_fund_size_and_hold_rate_query, _fund_size_and_hold_rate_filter_func)
-            _dt_index = self.dts.trading_days[self.dts.trading_days.datetime >= (self.start_date - datetime.timedelta(days=200))].datetime
-            _temp_size_and_hold_rate = _size_and_hold_rate.pivot(index='datetime', columns='fund_id').reindex(_dt_index).fillna(method='ffill')
+            _temp_size_and_hold_rate = _size_and_hold_rate.pivot(index='datetime', columns='fund_id')
+            _temp_size_and_hold_rate.index = [i.date() for i in _temp_size_and_hold_rate.index]
+            _dt_index = _temp_size_and_hold_rate.index.union(self.dts.trading_days[self.dts.trading_days.datetime >= (self.start_date - datetime.timedelta(days=200))].datetime)
+            _temp_size_and_hold_rate = _temp_size_and_hold_rate.reindex(_dt_index).fillna(method='ffill')
             self.dts.fund_size = _temp_size_and_hold_rate['size']
             self.dts.fund_com_hold = _temp_size_and_hold_rate['institution_holds']
             _tm_basic_fetch_fund_size = time.time()
@@ -344,6 +352,15 @@ class FundDataManager(DataManager):
                 print(f'\t[time][basic] fetch fund size: {_tm_basic_fetch_fund_size - _tm_basic_pivot_fund_nav}')
         _tm_basic = time.time()
 
+        # 连续四个季度可转债占比 大于20%
+        conv_df = self.dts.fund_conv_stats.pivot_table(index='datetime', values='conv_weights', columns='fund_id')
+        _date_list = conv_df.index.tolist()
+        conv_dict = {}
+        for idx, dt in enumerate(_date_list):
+            _df = conv_df.iloc[max(idx-4,0):(idx+1)]
+            fund_list = _df.columns[_df.min() > 20].values.tolist()
+            conv_dict[dt] = fund_list
+
         # get derived data first
         with DerivedDatabaseConnector().managed_session() as derived_session:
             _tm_deriv_start = time.time()
@@ -356,6 +373,10 @@ class FundDataManager(DataManager):
                 FundIndicator.track_err,
                 FundIndicator.year_length,
                 FundIndicator.info_ratio,
+                FundIndicator.alpha_bond,
+                FundIndicator.beta_bond,
+                FundIndicator.alpha_hs300,
+                FundIndicator.beta_hs300,
                 # FundIndicator.down_risk,
                 # FundIndicator.ret_over_period,
                 # FundIndicator.treynor,
@@ -410,16 +431,19 @@ class FundDataManager(DataManager):
                 FundManagerScore.model_beta,
                 FundManagerScore.sharpe,
                 FundManagerScore.fund_type,
+                FundManagerScore.fund_size,
             )
-            _fund_manager_score_query, _fund_manager_score_filter_func = _fund_manager_score_filter(_fund_manager_score_query)
+            _fund_manager_score_query, _fund_manager_score_filter_func = _fund_manager_score_filter(_fund_manager_score_query) 
             self.dts.fund_manager_score = self._s3_retriever.fetch_table(_fund_manager_score_query, _fund_manager_score_filter_func).drop(columns='fund_type')
-            self.dts.fund_manager_score = self.dts.fund_manager_score.pivot_table(index='datetime', columns='manager_id').reindex(trim_trading_days).stack().reset_index()
+            _dt =self.dts.trading_days[self.dts.trading_days.datetime >= self.dts.fund_manager_score.datetime.min()].datetime
+            self.dts.fund_manager_score = self.dts.fund_manager_score.pivot_table(index='datetime', columns='manager_id').reindex(_dt).fillna(method='ffill').reindex(trim_trading_days).stack().reset_index()
             _tm_deriv_apply_active_indicator = time.time()
             _fund_manager_info_query = derived_session.query(
                 FundManagerInfo.mng_id,
                 FundManagerInfo.start_date,
                 FundManagerInfo.fund_id,
                 FundManagerInfo.end_date,
+                FundManagerInfo.mng_name,
             )
             self.dts.fund_manager_info = self._s3_retriever.fetch_table(_fund_manager_info_query)
             self.dts.fund_manager_info = self.dts.fund_manager_info.set_index('mng_id')
@@ -442,12 +466,21 @@ class FundDataManager(DataManager):
             _df = _df.rename(columns = _mng_score_dic)
             _df['datetime'] = _df.datetime.dt.date
             self.dts.fund_indicator = pd.merge(self.dts.fund_indicator, _df,how='left',left_on=['fund_id','datetime'],right_on=['fund_id','datetime'])
-            #self.dts.fund_indicator['active'] = self.dts.fund_indicator['active'].fillna(0)
             _index_list = ['hs300','csi500','gem']
             
             self.dts.fund_indicator['active'] = self.dts.fund_indicator.fund_id.transform(lambda x: 0 if x in _normal_funds_index_dic and _normal_funds_index_dic[x] in _index_list else 1)
             self.dts.fund_indicator['index_id'] = self.dts.fund_indicator.fund_id.apply(lambda x: self.dts.fund_index_map.get(x, None))
+            _fund_indicator_whole = self.dts.fund_indicator.copy()
             self.dts.fund_indicator = self.dts.fund_indicator.dropna(subset=['index_id'])
+            conv_indicator_df = self.ipo_conv_indicator_process(conv_dict, 'conv_bond', _fund_indicator_whole)
+            self.dts.fund_indicator = self.dts.fund_indicator.append(conv_indicator_df)
+            self.dts.fund_indicator = pd.merge(self.dts.fund_indicator, ipo_weight_last_year_df, 
+                                                how='left', left_on=['fund_id','datetime'], right_on=['fund_id','datetime'])
+            self.dts.fund_indicator['ipo_weight'] = self.dts.fund_indicator['ipo_weight'].fillna(0)
+            _fund_size = self.dts.fund_size.stack().reset_index().rename(columns={0:'fund_size','level_0':'datetime'})
+            self.dts.fund_indicator = pd.merge(self.dts.fund_indicator, _fund_size, 
+                                            how='left', left_on=['fund_id','datetime'], right_on=['fund_id','datetime'])
+            self.dts.fund_indicator['fund_size'] = self.dts.fund_indicator['fund_size'].fillna(0)
             _tm_deriv_apply_manager_rank = time.time()
 
             if use_weekly_monthly_indicators:
@@ -547,6 +580,32 @@ class FundDataManager(DataManager):
             pass
         return dic
 
+    def ipo_conv_indicator_process(self, dic, _index_id, _fund_indicator_whole):
+        res = []
+        _date_list = np.array(sorted(list(dic.keys())))
+        for idx, dt in enumerate(_date_list):
+            if dt < self.start_date:
+                continue
+            if (idx + 1) == len(_date_list):
+                fund_list = dic[dt]
+                if len(fund_list) == 0:
+                    continue
+                _df = _fund_indicator_whole[(_fund_indicator_whole.fund_id.isin(fund_list))
+                                    & (_fund_indicator_whole.datetime >= dt)].copy()
+                _df['index_id'] = _index_id
+                res.append(_df)
+            else:
+                next_day = _date_list[idx + 1]
+                fund_list = dic[dt]
+                if len(fund_list) == 0:
+                    continue
+                _df = _fund_indicator_whole[(_fund_indicator_whole.fund_id.isin(fund_list))
+                                    & (_fund_indicator_whole.datetime >= dt)
+                                    & (_fund_indicator_whole.datetime < next_day)].copy()
+                _df['index_id'] = _index_id
+                res.append(_df)
+        return pd.concat(res)
+
     def _get_annual_alpha_beta(self, _fund_indicator_annual_fund_i, year):
         if year in _fund_indicator_annual_fund_i.index:
             return _fund_indicator_annual_fund_i.loc[year,'alpha_annual'], _fund_indicator_annual_fund_i.loc[year,'beta_annual']
@@ -634,22 +693,18 @@ class FundDataManager(DataManager):
                     FundNav.datetime <= self.end_date,
                 )
                 _nav_df = pd.read_sql(_fund_nav_query.statement, _fund_nav_query.session.bind)
+                if _nav_df is not None and not _nav_df.empty:
+                    # 这里我们制作一个总的pivot_table再取值，会比分别两次pivot_table快一些
+                    # 如果index-column pair有重复的值，pivot会报错而pivot_table不会，所以这里用pivot也许更好些
+                    # 但是pivot里边会调用set_index，导致datetime列的dtype从object变为datetime64[ns]
+                    _temp_fund_nav = _nav_df.pivot_table(index='datetime', columns='fund_id')
 
-                # 这里我们制作一个总的pivot_table再取值，会比分别两次pivot_table快一些
-                # 如果index-column pair有重复的值，pivot会报错而pivot_table不会，所以这里用pivot也许更好些
-                # 但是pivot里边会调用set_index，导致datetime列的dtype从object变为datetime64[ns]
-                _temp_fund_nav = _nav_df.pivot_table(index='datetime', columns='fund_id')
+                    self.dts.fund_nav = self.dts.fund_nav.join(_temp_fund_nav['adjusted_net_value'].dropna(axis=1, how='all'), how='outer').fillna(method='ffill')
+                    self.dts.fund_unit_nav = self.dts.fund_unit_nav.join(_temp_fund_nav['unit_net_value'].dropna(axis=1, how='all'), how='outer').fillna(method='ffill')
+                else:
+                    print(f'[fund_list_without_nav] no nav data of {fund_list_without_nav}')
 
-                self.dts.fund_nav = self.dts.fund_nav.join(
-                    _temp_fund_nav['adjusted_net_value'].dropna(axis=1, how='all').fillna(method='ffill'),
-                    how='outer'
-                )
-                self.dts.fund_unit_nav = self.dts.fund_unit_nav.join(
-                    _temp_fund_nav['unit_net_value'].dropna(axis=1, how='all').fillna(method='ffill'),
-                    how='outer'
-                )
-
-        print(f'[fund_list_without_nav] fund_ids added : {fund_list_without_nav}')
+        print(f'[fund_list_without_nav] fund_ids added: {fund_list_without_nav}')
         return fund_list_without_nav
 
     def get_fund_purchase_fees(self):
@@ -690,6 +745,20 @@ class FundDataManager(DataManager):
             BarraCNE5FactorProcessor.fund_analysis_plot(fund_ret, factor_ret_contri)
         return params, spec_ret
 
+    def fund_analysis_by_stock_factors(self, fund_id: str, factor_names: Tuple[str]) -> pd.DataFrame:
+        try:
+            try:
+                fund_nav = self.dts.fund_nav.loc[:, fund_id]
+            except KeyError:
+                print(f'[fund_analysis_new] do not have nav of {fund_id}, try to retrieve it')
+                if not self.confirm_fund_nav_exists(fund_id):
+                    return
+                fund_nav = self.dts.fund_nav.loc[:, fund_id]
+        except KeyError:
+            print(f'[fund_analysis_new] do not have nav of {fund_id} as before, can not do analysis')
+            return
+        return StockFactorApi.decomp_ret(fund_nav.pct_change(), factor_names)
+
     def white_and_black_list_filter(self, score, score_raw, disproved_list):
         return self._score_manager._white_and_black_list_filter(score, score_raw, disproved_list)
 
