@@ -52,12 +52,19 @@ class ScoreFunc:
     track_err: float = 0  # weight of track_err
     fee_rate: float = 0  # weight of fee_rate
     info_ratio: float = 0
+    ipo_weight: float = 0
+    fund_size: float = 0
     mng_annualized_ret: float = 0
     mng_annualized_vol: float = 0
     mng_model_alpha: float = 0
     mng_model_beta: float = 0
     mng_sharpe: float = 0
     mng_trade_year: float = 0
+    mng_fund_size: float = 0
+    alpha_bond: float = 0
+    beta_bond: float = 0
+    alpha_hs300: float = 0
+    beta_hs300: float = 0
     active: float = -10 #默认条件不选择主动型，对主动性加分数惩罚
     # last_year_alpha: float = 0
     # last_year_beta: float = 0
@@ -89,12 +96,19 @@ class ScoreFunc:
                 + self.fee_rate * data.fee_rate \
                 + self.beta * abs(1-data.beta) \
                 + self.info_ratio * data.info_ratio \
+                + self.alpha_bond * data.alpha_bond \
+                + self.beta_bond * data.beta_bond \
+                + self.alpha_hs300 * data.alpha_hs300 \
+                + self.beta_hs300 * data.beta_hs300 \
+                + self.ipo_weight * data.ipo_weight \
+                + self.fund_size * data.fund_size \
                 + data.mng_annualized_ret.mul(other=self.mng_annualized_ret, fill_value=0) \
                 + data.mng_annualized_vol.mul(other=self.mng_annualized_vol, fill_value=0) \
                 + data.mng_model_alpha.mul(other=self.mng_model_alpha, fill_value=0) \
                 + data.mng_model_beta.mul(other=self.mng_model_beta, fill_value=0) \
                 + data.mng_sharpe.mul(other=self.mng_sharpe, fill_value=0) \
                 + data.mng_trade_year.mul(other=self.mng_trade_year, fill_value=0) \
+                + data.mng_fund_size.mul(other=self.mng_fund_size, fill_value=0) \
                 + data.active * self.active
                 # + self.down_risk * data.down_risk \
                 # + self.info_ratio * data.info_ratio \
@@ -162,7 +176,7 @@ class FundScoreManager:
             'hs300': {
                 'default': ScoreFunc(track_err=-0.4,beta=-0.6),
                 'enhanced':ScoreFunc(alpha=0.6, beta=-0.3),
-                'active': ScoreFunc(mng_trade_year=0.15, mng_annualized_ret=0.15, mng_sharpe=0.15,alpha=0.1,mng_model_alpha=0.3,mng_model_beta=0.15,active=10),    
+                'active': ScoreFunc(mng_trade_year=0.15, mng_annualized_ret=0.15,alpha=0.3,mng_model_alpha=0.3,mng_model_beta=0.1,active=10),
             },
             'csi500': {
                 'default': ScoreFunc(info_ratio=0.2,track_err=-0.7, fee_rate=-0.1),
@@ -192,6 +206,9 @@ class FundScoreManager:
             'active': {
                 'default':ManagerFunc(trade_year=0.15, annualized_ret=0.15, sharpe=0.15, model_alpha=0.3, model_beta=0.15)
             },
+            'conv_bond':{
+                'default':ScoreFunc(fee_rate=-0.1, alpha_bond=0.4, beta_bond=-0.1),# set default score equation
+            }
         }
         self.score_cache = defaultdict(dict)
         self.score_raw_cache = defaultdict(dict)
@@ -335,6 +352,7 @@ class FundScoreManager:
             cur_d_sta = cur_d.apply(lambda x: (x - x.mean() + 1e-6)/ (x.std() + 1e-6), axis=0)
             cur_d_sta['beta'] = cur_d['beta']
             cur_d_sta['year_length'] = cur_d['year_length']
+            cur_d_sta['active'] = cur_d['active']
             cur_d = cur_d_sta
             score_raw_sum = {}
             score_sum = {}
