@@ -108,6 +108,8 @@ class FundStatusLatest(Base):
     OTCLPLIMITJG = Column('otc_lp_limit_jg', DOUBLE(asdecimal=False))  # 场外暂停大额申购上限(机构)
     OTCLPLIMITGR = Column('otc_lp_limit_gr', DOUBLE(asdecimal=False))  # 场外暂停大额申购上限(个人)
     TRADESTATUS = Column('trade_status', CHAR(16))  # 交易状态
+    PCHMINAMT = Column('pch_min_amt', DOUBLE(asdecimal=False))  # 申购金额下限
+    REDMMINAMT = Column('redm_min_amt', DOUBLE(asdecimal=False))  # 单笔赎回份额下限
 
 
 class FundInfo(Base):
@@ -124,6 +126,7 @@ class FundInfo(Base):
     end_date = Column(DATE) # 关闭日期
     wind_class_1 = Column(CHAR(64)) # Wind基金类型
     wind_class_2 = Column(CHAR(64)) # Wind基金二级类型
+    risk_level = Column(CHAR(2))  # 风险等级
     manager_id = Column(TEXT) # 基金经理
     company_id = Column(CHAR(64)) # 基金公司
     benchmark = Column(CHAR(255)) # 业绩基准
