@@ -1,7 +1,9 @@
-from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN, text, TEXT
+from sqlalchemy import CHAR, Column, Integer, Index, BOOLEAN, text, TEXT, Enum
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.dialects.mysql import DOUBLE, DATE, DATETIME, SMALLINT
 
+from ...constant import StockFactorType
+
 
 class Base():
     _update_time = Column('_update_time', DATETIME, nullable=False, server_default=text('CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP'))  # 更新时间
@@ -121,7 +123,6 @@ class IndexValuationLongTerm(Base):
         Index('idx_index_valuation_long_term_datetime', 'datetime'),
     )
 
-
 class FundIndicator(Base):
     '''基金评价指标'''
 
@@ -147,6 +148,10 @@ class FundIndicator(Base):
     r_square = Column(DOUBLE(asdecimal=False)) #决定系数R方
     sharpe = Column(DOUBLE(asdecimal=False)) #夏普率
     year_length = Column(DOUBLE(asdecimal=False)) #成立年限
+    alpha_bond = Column(DOUBLE(asdecimal=False)) #alpha 债
+    beta_bond = Column(DOUBLE(asdecimal=False)) #beta 债
+    alpha_hs300 = Column(DOUBLE(asdecimal=False)) #alpha hs300
+    beta_hs300 = Column(DOUBLE(asdecimal=False)) #beta hs300
 
     __table_args__ = (
         Index('idx_fund_indicator_datetime', 'datetime'),
@@ -252,33 +257,11 @@ class FundIndicatorGroup(Base):
     stock_hm_beta = Column(DOUBLE(asdecimal=False))  # hm beta
     stock_hm_beta_t = Column(DOUBLE(asdecimal=False))  # hm beta t值
     stock_hm_beta_p = Column(DOUBLE(asdecimal=False))  # hm beta p值
-
+    abnormal_reason = Column(CHAR(60)) # 未计算因子 异常原因
     __table_args__ = (
         Index('idx_fund_indicator_group_datetime_data_cycle', 'datetime', 'data_cycle'),
     )
 
-
-class FundScore(Base):
-    '''基金评分'''
-
-    __tablename__ = 'fund_score'
-    id = Column(Integer, primary_key=True)
-
-    fund_id = Column(CHAR(10)) # 基金ID
-    datetime = Column(DATE) # 日期
-    score = Column(DOUBLE(asdecimal=False)) # 评分
-    tag_name = Column(CHAR(64)) # 基金类别
-    tag_type = Column(Integer) # 1 '大类资产评分'
-    tag_method = Column(CHAR(40)) # 打标签版本号
-    is_full = Column(BOOLEAN) # 1 true
-    score_method = Column(Integer)
-    desc_name = Column(CHAR(64))
-
-    __table_args__ = (
-        Index('idx_fund_score_datetime', 'datetime'),
-    )
-
-
 class FundScoreExtended(Base):
     '''新基金评分v1'''
 
@@ -340,6 +323,7 @@ class FundScoreNew(Base):
     selection_score = Column(DOUBLE(asdecimal=False))  # 被动指数型、被动指数型债券(均为二级分类)基金和货基(一二级分类相同)为机构观点，其他为选证能力
     team_score = Column(DOUBLE(asdecimal=False))  # 基金公司团队能力
     total_score = Column(DOUBLE(asdecimal=False), nullable=False)  # 总分
+    abnormal_reason = Column(CHAR(60)) # 未计算因子 异常原因
 
     __table_args__ = (
         Index('idx_fund_score_new_data_cycle', 'data_cycle'),
@@ -464,7 +448,6 @@ class StyleReg(Base):
     small_g = Column(DOUBLE(asdecimal=False))
     small_v = Column(DOUBLE(asdecimal=False))
 
-
 class FundManagerIndex(Base):
     '''基金经理指数'''
 
@@ -491,8 +474,11 @@ class FundManagerScore(Base):
     alpha = Column(DOUBLE(asdecimal=False))
     beta = Column(DOUBLE(asdecimal=False))
     track_err = Column(DOUBLE(asdecimal=False))
+    fund_size = Column(DOUBLE(asdecimal=False))
     model_alpha = Column(DOUBLE(asdecimal=False))
     model_beta = Column(DOUBLE(asdecimal=False))
+    now_comp_year = Column(DOUBLE(asdecimal=False)) # 在当前基金公司时间
+    year_ret_std =  Column(DOUBLE(asdecimal=False)) # 每年收益波动率
     score = Column(DOUBLE(asdecimal=False))
 
     __table_args__ = (
@@ -587,6 +573,9 @@ class ThirdPartyPortfolioPositionLatest(Base):
     po_src = Column(SMALLINT, nullable=False)  # 组合来源 详见constant.SourceType
     position = Column(TEXT, nullable=False)  # 持仓
 
+    __table_args__ = (
+        Index('idx_third_party_portfolio_position_latest_po_src', 'po_src'),
+    )
 
 class ThirdPartyPortfolioIndicator(Base):
     '''组合最新因子表'''
@@ -660,4 +649,14 @@ class TopManagerFunds(Base):
     top_7 = Column(CHAR(10))  # 第七名基金
     top_8 = Column(CHAR(10))  # 第八名基金
     top_9 = Column(CHAR(10))  # 第九名基金
-    top_10 = Column(CHAR(10)) # 第十名基金
\ No newline at end of file
+    top_10 = Column(CHAR(10)) # 第十名基金
+
+class StockFactorInfo(Base):
+    '''因子信息表'''
+
+    __tablename__ = 'stock_factor_info'
+
+    f_id = Column(CHAR(16), primary_key=True)  # 因子ID
+    f_type = Column(Enum(StockFactorType), nullable=False)  # 因子所属分类
+    f_name = Column(TEXT, nullable=False)  # 因子名称
+    f_desc = Column(TEXT)  # 因子简介
