@@ -501,6 +501,8 @@ class EmFundScale(Base):
     OTCLPLIMITJG = Column(DOUBLE(asdecimal=False))  # 场外暂停大额申购上限(机构)
     OTCLPLIMITGR = Column(DOUBLE(asdecimal=False))  # 场外暂停大额申购上限(个人)
     TRADESTATUS = Column(CHAR(16))  # 交易状态
+    PCHMINAMT = Column(DOUBLE(asdecimal=False))  # 申购金额下限
+    REDMMINAMT = Column(DOUBLE(asdecimal=False))  # 单笔赎回份额下限
 
 
 class EMIndexPct(Base):
@@ -568,12 +570,15 @@ class EmStockDailyInfo(Base):
     CODES = Column("stock_id", CHAR(10), primary_key=True) # EM股票ID
     DATES = Column('datetime', DATE, primary_key=True) # 日期
     TOTALSHARE = Column("total_share", DOUBLE(asdecimal=False), nullable=False) # 总股本
+    LIQSHARE = Column("liq_share", DOUBLE(asdecimal=False), nullable=False) # 流通股本
     HOLDFROZENAMTACCUMRATIO = Column('hold_frozen_amt_accum_ratio', DOUBLE(asdecimal=False))  # 控股股东累计质押数量占持股比例
     PETTMDEDUCTED = Column('pe_ttm_deducted', DOUBLE(asdecimal=False))  # 市盈率TTM(扣除非经常性损益)
     PBLYRN = Column('pb_lyr_n', DOUBLE(asdecimal=False))  # 市净率(PB，LYR)(按公告日)
     PSTTM = Column('ps_ttm', DOUBLE(asdecimal=False))  # 市销率(PS，TTM)
     AHOLDER = Column('a_holder', TEXT)  # 实际控制人
     ESTPEG = Column('est_peg', DOUBLE(asdecimal=False)) # 预测PEG(最近一年)
+    SHAREHDPCT = Column('share_hd_pct', DOUBLE(asdecimal=False)) # 沪深港通持股比例
+    EVWITHOUTCASH = Column('ev_without_cash', DOUBLE(asdecimal=False)) # 企业价值(剔除货币资金)(EV2)
 
     __table_args__ = (
         Index('idx_em_stock_daily_info_datetime', 'datetime'),
@@ -595,8 +600,8 @@ class EmStockInfo(Base):
     STOHSTOCKCONNECTEDORNOT = Column("stoh_stock_connected_ornot", BOOLEAN, nullable=False) # 是否属于沪股通标的
     SHENGUTONGTARGET = Column("shengutong_target", BOOLEAN, nullable=False) # 是否属于深股通标的
     BLEMINDCODE = Column("bl_em_ind_code", CHAR(40)) # 所属东财行业指数代码
-    BLSWSINDCODE = Column("bl_sws_ind_code", CHAR(40), nullable=False) # 所属申万行业指数代码
-    SW2014CODE = Column("sw_2014_code", CHAR(40), nullable=False) # 所属申万行业代码
+    BLSWSINDCODE = Column("bl_sws_ind_code", CHAR(40)) # 所属申万行业指数代码
+    SW2014CODE = Column("sw_2014_code", CHAR(40)) # 所属申万行业代码
     EMINDCODE2016 = Column("em_ind_code_2016", CHAR(40), nullable=False) # 所属东财行业(2016)代码
     CITICCODE2020 = Column("citic_code_2020", CHAR(40)) # 所属中信行业代码(2020)
     BLCSRCINDCODE = Column("bl_csrc_ind_code", CHAR(32)) # 所属证监会行业指数代码
@@ -680,7 +685,9 @@ class EmStockFinFac(Base):
     BPS = Column('bps', DOUBLE(asdecimal=False)) # 每股净资产
     MBREVENUE = Column('mb_revenue', DOUBLE(asdecimal=False)) # 主营业务收入
     MBCOST = Column('mb_cost', DOUBLE(asdecimal=False)) # 主营业务支出
+    BALANCESTATEMENT_9 = Column('balance_statement_9', DOUBLE(asdecimal=False)) # 货币资金
     BALANCESTATEMENT_25 = Column('balance_statement_25', DOUBLE(asdecimal=False)) # 流动资产合计
+    BALANCESTATEMENT_31 = Column('balance_statement_31', DOUBLE(asdecimal=False)) # 固定资产
     BALANCESTATEMENT_46 = Column('balance_statement_46', DOUBLE(asdecimal=False)) # 非流动资产合计
     BALANCESTATEMENT_74 = Column('balance_statement_74', DOUBLE(asdecimal=False)) # 资产总计
     BALANCESTATEMENT_93 = Column('balance_statement_93', DOUBLE(asdecimal=False)) # 流动负债合计
@@ -688,6 +695,7 @@ class EmStockFinFac(Base):
     BALANCESTATEMENT_128 = Column('balance_statement_128', DOUBLE(asdecimal=False)) # 负债合计
     BALANCESTATEMENT_140 = Column('balance_statement_140', DOUBLE(asdecimal=False)) # 归属于母公司股东权益合计
     BALANCESTATEMENT_141 = Column('balance_statement_141', DOUBLE(asdecimal=False)) # 股东权益合计
+    BALANCESTATEMENT_141_ADJ = Column('balance_statement_141_adj', DOUBLE(asdecimal=False)) # 股东权益合计(调整)
     BALANCESTATEMENT_195 = Column('balance_statement_195', DOUBLE(asdecimal=False)) # 其他权益工具
     BALANCESTATEMENT_196 = Column('balance_statement_196', DOUBLE(asdecimal=False)) # 其中:优先股(其他权益工具)
     INCOMESTATEMENT_9 = Column('income_statement_9', DOUBLE(asdecimal=False)) # 营业收入
@@ -698,6 +706,7 @@ class EmStockFinFac(Base):
     INCOMESTATEMENT_83 = Column('income_statement_83', DOUBLE(asdecimal=False)) # 营业总收入
     INCOMESTATEMENT_85 = Column('income_statement_85', DOUBLE(asdecimal=False)) # 其他业务收入
     INCOMESTATEMENT_127 = Column('income_statement_127', DOUBLE(asdecimal=False)) # 利息费用
+    CASHFLOWSTATEMENT_9 = Column('cashflow_statement_9', DOUBLE(asdecimal=False)) # 销售商品、提供劳务收到的现金
     CASHFLOWSTATEMENT_39 = Column('cashflow_statement_39', DOUBLE(asdecimal=False)) # 经营活动产生的现金流量净额
     CASHFLOWSTATEMENT_59 = Column('cashflow_statement_59', DOUBLE(asdecimal=False)) # 投资活动产生的现金流量净额
     CASHFLOWSTATEMENT_77 = Column('cashflow_statement_77', DOUBLE(asdecimal=False)) # 筹资活动产生的现金流量净额
@@ -817,6 +826,7 @@ class EmFundInfo(Base):
     MATURITYDATE = Column('end_date', DATE) # 结束日期
     FIRSTINVESTTYPE = Column('invest_type_class_1', CHAR(32), nullable=False) # 投资类型（一级）
     SECONDINVESTTYPE = Column('invest_type_class_2', CHAR(32), nullable=False) # 投资类型（二级）
+    RISKLEVEL = Column('risk_level', CHAR(2)) # 风险等级
     TRADECUR = Column('currency', CHAR(16), nullable=False) # 交易币种
     STRUCFUNDORNOT = Column('is_structured', BOOLEAN, nullable=False) # 是否分级基金
     GFCODEM = Column('base_fund_id', CHAR(16)) # 分级基金母基金代码
@@ -1314,6 +1324,7 @@ class EmMacroEconomicDaily(Base):
     # SHIBOR_1M: 1个月SHIBOR
     # SHIBOR_3M: 3个月SHIBOR
     # CB_AAA_1Y: 交易所公司债到期收益率(AAA):1年
+    # EX_DR_RATIO: 超额存款准备金率：日
     codes = Column(CHAR(12), primary_key=True)
     datetime = Column(DATE, primary_key=True) # 日期
 
@@ -1477,3 +1488,13 @@ class EmStockResearchInfo(Base):
     RESERCHSECUNUM = Column('research_secu_num', DOUBLE(asdecimal=False))  # 证券公司调研家数
     RESERCHSECUNAME = Column('research_ecu_name', TEXT)  # 调研的证券公司
     RESERCHSECUFREQUENCY1 = Column('research_secu_freq1', DOUBLE(asdecimal=False))  # 证券公司调研频次
+
+
+class EmStockYearly(Base):
+    '''Choice股票年度数据'''
+
+    __tablename__ = 'em_stock_yearly'
+
+    CODES = Column('stock_id', CHAR(10), primary_key=True)  # EM股票ID
+    year = Column(YEAR, primary_key=True)  # 年度
+    DIVANNUACCUM = Column('div_annu_accum', DOUBLE(asdecimal=False))  # 年度累计分红
