@@ -70,6 +70,7 @@ class FundDailyCollection(Base):
     group_num = Column(Integer)  # 同类数量
 
     fund_manager = Column(CHAR(255))  # 基金经理
+    fund_manager_id = Column(CHAR(127))  # 基金经理ID
     company_name = Column(CHAR(64))  # 基金公司
     symbol = Column(CHAR(64)) # 基金名称
     benchmark = Column(CHAR(255)) # 业绩基准
@@ -170,6 +171,7 @@ class FundDailyCollection(Base):
             'vol': '成立以来波动率',
             'max_drop_down': '最大回撤',
             'fund_manager': '基金经理',
+            'fund_manager_id': '基金经理ID',
             'company_name': '基金公司',
             'benchmark': '业绩基准',
             'benchmark_s': '跟踪标的ID',
