@@ -103,7 +103,7 @@ class FundEngine:
         self._prep_manager_score, self._prep_manager_funds, self._prep_manager_score_cleaned = self._dm.get_manager_scores(dt, self.score_select)
         self._prep_fund_score['active'] = self._prep_manager_score
         self._prep_fund_score_raw['active'] = self._prep_manager_score
-            
+        self._prep_fund_end_date_dict = self._dm.dts.fund_end_date_dict  
         if self.disproved_set:
             self._prep_fund_score, self._prep_fund_score_raw = self._dm.white_and_black_list_filter(self._prep_fund_score, self._prep_fund_score_raw, self.disproved_set)
         # ALLOCATION
@@ -133,7 +133,7 @@ class FundEngine:
                 dt, self._prep_target_fund_allocation, self._prep_target_asset_allocation,
                 cur_fund_position, cur_asset_position, 
                 self._prep_fund_nav, self._prep_fund_score, self._prep_asset_price,self._prep_fund_unit_nav, 
-                is_real_trade, self._prep_manager_funds, self.score_select
+                is_real_trade, self._prep_manager_funds, self.score_select, self._prep_fund_end_date_dict
             )
             self._report_helper.update_rebalance_detail(dt, trigger_reason)
             return fund_trade_list
@@ -254,6 +254,7 @@ class BasicFundBacktestEngine(object):
     def __init__(self, data_manager:FundDataManager, trader, fund_weight_list:list):
         self._dm = data_manager
         fund_ids = []
+        self.repair_trade_date(fund_weight_list)
         for dt, fund_weight in fund_weight_list:
             for fund_id, weight in fund_weight:
                 fund_ids.append(fund_id)
@@ -278,6 +279,21 @@ class BasicFundBacktestEngine(object):
             cur.fund_id: (wind_index_map[cur.wind_class_1] if cur.wind_class_1 in wind_index_map else 'hsi') for cur in self._dm.dts.fund_info.itertuples()
         }
 
+    def repair_trade_date(self, fund_weight_list):
+        _dts = self._dm.get_trading_days().datetime
+        i = 0
+        for dt in _dts:
+            if i >= len(fund_weight_list):
+                break
+            cur_date = datetime.datetime.strptime(fund_weight_list[i][0], '%Y-%m-%d').date()
+            if dt < cur_date:
+                continue
+            if dt == cur_date:
+                i += 1
+                continue
+            if dt > cur_date:
+                fund_weight_list[i][0] = datetime.date.strftime(dt, '%Y-%m-%d')
+                i += 1
 
     def init(self):
         if not self._dm.inited:
@@ -415,6 +431,124 @@ class BasicFundBacktestEngine(object):
     def get_last_target_fund_allocation(self):
         return self._report_helper.get_last_target_fund_allocation(self._prep_target_fund_allocation.funds)
 
+
+class AIPFundBackTestEngine(BasicFundBacktestEngine):
+
+    DEFAULT_CASH_VALUE = _DEFAULT_CASH_VALUE
+
+    def __init__(self, data_manager: FundDataManager, trader, fund_weight_list: list):
+        self._dm = data_manager
+        self.repair_trade_date(fund_weight_list)
+        fund_ids = []
+        for dt, cash, fund_weight in fund_weight_list:
+            for fund_id, weight in fund_weight:
+                fund_ids.append(fund_id)
+        self._dm.confirm_fund_nav_exists(fund_ids)
+        self._report_helper = ReportHelper()
+        self._pending_trades = []
+        self._trader = trader
+        self._new_fund_weight = {}
+        self._new_fund_cash = {}
+        self.cur_fund_position = None
+        self.cur_asset_position = AssetPosition(cash=1)
+        self.deposits = []
+        self.current_deposit = None
+
+        for dt, cash, fund_weight in fund_weight_list:
+            dt = dt if isinstance(dt, datetime.date) else datetime.datetime.strptime(dt, '%Y-%m-%d').date()
+            self._new_fund_weight[dt] = fund_weight
+            self._new_fund_cash[dt] = cash
+
+        wind_index_map = {
+            '货币市场型基金': 'mmf',
+            '股票型基金': 'hs300',
+            '混合型基金': 'csi500',     # TODO: 临时代替，需设置正确映射关系！！！ 抄袭上面
+            '国际(QDII)基金': 'sp500rmb',
+            '另类投资基金': 'gold',
+            '债券型基金': 'national_debt',
+            '其他基金': 'hsi'           # TODO: 临时代替，需设置正确映射关系！！！
+        }
+        self._fund_wind_class = {
+            cur.fund_id: (wind_index_map[cur.wind_class_1] if cur.wind_class_1 in wind_index_map else 'hsi') for cur in self._dm.dts.fund_info.itertuples()
+        }
+
+    def run(self, start_date: datetime.date = None, end_date: datetime.date = None, cash: float = None, print_time=False):
+        # setup helpers
+        self.setup()
+
+        # init days
+        start_date = start_date or self._dm.start_date
+        end_date = end_date or self._dm.end_date
+        start_date = start_date if isinstance(start_date, datetime.date) else datetime.datetime.strptime(start_date,'%Y%m%d').date()
+        end_date = end_date if isinstance(end_date, datetime.date) else datetime.datetime.strptime(end_date,'%Y%m%d').date()
+
+        # loop trading days
+        _dts = self._dm.get_trading_days()
+        dts = _dts[(_dts.datetime >= start_date) & (_dts.datetime <= end_date)].datetime # df with datetime.date
+        for t in dts:
+            self._run_on(t, print_time=print_time)
+        # init report data
+        self._report_helper.plot_init(self._dm)
+
+    def _run_on(self, dt, print_time=False, cash=None):
+        _tm_start = time.time()
+        # prep data
+        self.prep_data(dt)
+        _tm_prep_data = time.time()
+        # finalize trade
+        if self.cur_fund_position:
+            traded_list = self.finalize_trade(dt, self._pending_trades, self.cur_asset_position, self.cur_fund_position)
+        else:
+            traded_list = []
+        _tm_finalize_trade = time.time()
+        # calc trade
+        trade_list = self.calc_trade(dt, self.cur_asset_position, self.cur_fund_position)
+        _tm_calc_trade = time.time()
+        # book trade
+        self.book_trades(trade_list)
+        # update
+        self.update_reporter(dt, traded_list, self.cur_asset_position, self.cur_fund_position)
+        self.current_deposit = None
+        if self._pending_trades:
+            print(self._pending_trades)
+        _tm_finish = time.time()
+
+        if print_time:
+            print(f'{dt} (tot){_tm_finish - _tm_start} (finalize){_tm_finalize_trade - _tm_start} (calc){_tm_calc_trade - _tm_finalize_trade} (misc){_tm_finish - _tm_calc_trade}')
+
+    def calc_trade(self, dt, cur_asset_position: AssetPosition, cur_fund_position: FundPosition=None, is_real_trade=False):
+        if dt not in self._new_fund_weight:
+            return []
+
+        curr_target_fund_weight = FundWeight()
+        for fund_id, weight in self._new_fund_weight[dt]:
+            index_id = self._fund_wind_class[fund_id]
+
+            curr_target_fund_weight.add(FundWeightItem(fund_id=fund_id, fund_wgt=weight,
+                                                       index_id=index_id, asset_wgt=0, fund_wgt_in_asset=0))
+
+        cash = self._new_fund_cash[dt]
+        new_fund_position = FundPosition(cash=cash)
+        if not self.cur_fund_position:
+            self.cur_fund_position = new_fund_position
+        else:
+            self.cur_fund_position.cash = self.cur_fund_position.cash + cash
+        self.deposits.append(cash)
+        self.current_deposit = cash
+
+        self._report_helper.update_rebalance_detail(dt, 'Change fund weight')
+        v_fund_position, fund_trade_list = self._trader.calc_fund_trade(
+            dt, curr_target_fund_weight, new_fund_position, self._prep_fund_nav,
+            self._prep_fund_purchase_fees, self._prep_fund_redeem_fees)
+        return fund_trade_list
+
+    def update_reporter(self, dt, trade_list, cur_asset_position: AssetPosition,
+                        cur_fund_position: FundPosition = None, current_deposit = None):
+        self._report_helper.update(dt, cur_asset_position, self._prep_asset_price, self._pending_trades,
+                                   cur_fund_position, self._prep_fund_nav, trade_list, None, None, None,
+                                   current_deposit=current_deposit)
+
+
 def saa_backtest(m: FundDataManager, saa: AssetWeight):
     asset_param = AssetTradeParam() # type in here
     t = AssetTrader(asset_param)
@@ -787,6 +921,7 @@ params_str_1 = '''
     "fund_weight_list": [
         [
             "2020-08-05",
+            1000,
             [
                 [
                     "000001!0",
@@ -803,7 +938,7 @@ params_str_1 = '''
 def test1():
     import json
     import pickle
-    with open('/shared/robo-advisor/temp/data_manager', 'rb') as fp:
+    with open('/Users/puyuantech/Downloads/robo-advisor/temp/data_manager', 'rb') as fp:
         data_manager = pickle.load(fp)
 
     t = BasicFundTrader()
@@ -816,7 +951,7 @@ def test1():
     start_date = datetime.datetime.strptime(start_date, '%Y-%m-%d').date()
     end_date = datetime.datetime.strptime(end_date, '%Y-%m-%d').date()
 
-    b = BasicFundBacktestEngine(data_manager=data_manager, trader=t, fund_weight_list=fund_weight_list)
+    b = AIPFundBackTestEngine(data_manager=data_manager, trader=t, fund_weight_list=fund_weight_list)
     b.init()
     # print(b._dm.dts.fund_index_map)
     b.run(start_date=start_date, end_date=end_date, print_time=True)
