@@ -170,6 +170,9 @@ class ReportHelper:
         w['last_increase_rate'] = res.last_increase_rate
         w['market_value'] = df #dataframe
         w['rebalance_times'] = len(self.rebalance_date)
+        w['recent_drawdown'] = res.recent_drawdown
+        w['worst_3m_ret'] = res.worst_3m_ret
+        w['worst_6m_ret'] = res.worst_6m_ret
         hold_days = self.rebalance_date + [df.index.values[-1]]
         index_list = df.index.tolist()
         result = {}
@@ -697,14 +700,14 @@ class ReportHelper:
         return res_df[['begin_date','end_date','desc_name','index_id','weight','rets']]
 
     @staticmethod
-    def rolling_analysis(rolling_year, index_id, m, mv_df, fund_annual_ret):
+    def rolling_analysis(rolling_year, index_id, m, mv_df, fund_annual_ret, ret_list=[]):
         if '沪深300' in mv_df:
             mv_df = mv_df.drop('沪深300', axis=1)
         mv_df = mv_df.join(m.dts.index_price[[index_id]])
         mv_df = mv_df / mv_df.shift(1)
         annual_ret = (mv_df.rolling(window=242*rolling_year).agg(lambda x : x.prod() - 1))/rolling_year
-        FundPainter.plot_rolling_analysis(annual_ret,fund_annual_ret,rolling_year,index_id)
-    
+        FundPainter.plot_rolling_analysis(annual_ret,fund_annual_ret,rolling_year,index_id,ret_list)
+     
     @staticmethod
     def regular_rebalance_in_index_strategy(m, begin_date,end_date,index_id,rebalance_year,top_funds):
         dts = m.dts.trading_days.datetime
