@@ -30,9 +30,9 @@ def xirr(values, dates):
     0.0100612...
     '''
     try:
-        return scipy.optimize.brentq(lambda r: xnpv(r, values, dates), -0.999, 1e10)
-    except RuntimeError:    # Failed to converge?
-        return scipy.optimize.newton(lambda r: xnpv(r, values, dates),  x0=0.0, maxiter=40)
+        return scipy.optimize.brentq(lambda r: xnpv(r, values, dates), -0.999, 1e3)
+    except Exception:    # Failed to converge?
+        return scipy.optimize.bisect(lambda r: xnpv(r, values, dates), -0.999, 1e3)
 
 
 class AIPCalculator(object):
